# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os.path as op

from ...utils.filemanip import split_filename
from ..base import (CommandLineInputSpec, CommandLine, traits, TraitedSpec,
                    File, InputMultiPath, isdefined)


class MRConvertInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='voxel-order data filename')
    out_filename = File(
        genfile=True, argstr='%s', position=-1, desc='Output filename')
    extract_at_axis = traits.Enum(
        1,
        2,
        3,
        argstr='-coord %s',
        position=1,
        desc=
        '"Extract data only at the coordinates specified. This option specifies the Axis. Must be used in conjunction with extract_at_coordinate.'
    )
    extract_at_coordinate = traits.List(
        traits.Float,
        argstr='%s',
        sep=',',
        position=2,
        minlen=1,
        maxlen=3,
        desc=
        '"Extract data only at the coordinates specified. This option specifies the coordinates. Must be used in conjunction with extract_at_axis. Three comma-separated numbers giving the size of each voxel in mm.'
    )
    voxel_dims = traits.List(
        traits.Float,
        argstr='-vox %s',
        sep=',',
        position=3,
        minlen=3,
        maxlen=3,
        desc=
        'Three comma-separated numbers giving the size of each voxel in mm.')
    output_datatype = traits.Enum(
        "nii",
        "float",
        "char",
        "short",
        "int",
        "long",
        "double",
        argstr='-output %s',
        position=2,
        desc=
        '"i.e. Bfloat". Can be "char", "short", "int", "long", "float" or "double"'
    )  # , usedefault=True)
    extension = traits.Enum(
        "mif",
        "nii",
        "float",
        "char",
        "short",
        "int",
        "long",
        "double",
        position=2,
        desc=
        '"i.e. Bfloat". Can be "char", "short", "int", "long", "float" or "double"',
        usedefault=True)
    layout = traits.Enum(
        "nii",
        "float",
        "char",
        "short",
        "int",
        "long",
        "double",
        argstr='-output %s',
        position=2,
        desc=
        'specify the layout of the data in memory. The actual layout produced will depend on whether the output image format can support it.'
    )
    resample = traits.Float(
        argstr='-scale %d',
        position=3,
        units='mm',
        desc='Apply scaling to the intensity values.')
    offset_bias = traits.Float(
        argstr='-scale %d',
        position=3,
        units='mm',
        desc='Apply offset to the intensity values.')
    replace_NaN_with_zero = traits.Bool(
        argstr='-zero', position=3, desc="Replace all NaN values with zero.")
    prs = traits.Bool(
        argstr='-prs',
        position=3,
        desc=
        "Assume that the DW gradients are specified in the PRS frame (Siemens DICOM only)."
    )


class MRConvertOutputSpec(TraitedSpec):
    converted = File(exists=True, desc='path/name of 4D volume in voxel order')


class MRConvert(CommandLine):
    """
    Perform conversion between different file types and optionally extract a subset of the input image.

    If used correctly, this program can be a very useful workhorse.
    In addition to converting images between different formats, it can
    be used to extract specific studies from a data set, extract a specific
    region of interest, flip the images, or to scale the intensity of the images.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> mrconvert = mrt.MRConvert()
    >>> mrconvert.inputs.in_file = 'dwi_FA.mif'
    >>> mrconvert.inputs.out_filename = 'dwi_FA.nii'
    >>> mrconvert.run()                                 # doctest: +SKIP
    """

    _cmd = 'mrconvert'
    input_spec = MRConvertInputSpec
    output_spec = MRConvertOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['converted'] = self.inputs.out_filename
        if not isdefined(outputs['converted']):
            outputs['converted'] = op.abspath(self._gen_outfilename())
        else:
            outputs['converted'] = op.abspath(outputs['converted'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        if isdefined(self.inputs.out_filename):
            outname = self.inputs.out_filename
        else:
            outname = name + '_mrconvert.' + self.inputs.extension
        return outname


class DWI2TensorInputSpec(CommandLineInputSpec):
    in_file = InputMultiPath(
        File(exists=True),
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Diffusion-weighted images')
    out_filename = File(
        name_template="%s_tensor.mif",
        name_source="in_file",
        output_name="tensor",
        argstr='%s',
        desc='Output tensor filename',
        position=-1)
    encoding_file = File(
        argstr='-grad %s',
        position=2,
        desc=('Encoding file supplied as a 4xN text file with '
              'each line is in the format [ X Y Z b ], where '
              '[ X Y Z ] describe the direction of the applied '
              'gradient, and b gives the b-value in units '
              '(1000 s/mm^2). See FSL2MRTrix()'))
    ignore_slice_by_volume = traits.List(
        traits.Int,
        argstr='-ignoreslices %s',
        sep=' ',
        position=2,
        minlen=2,
        maxlen=2,
        desc=('Requires two values (i.e. [34 '
              '1] for [Slice Volume] Ignores '
              'the image slices specified '
              'when computing the tensor. '
              'Slice here means the z '
              'coordinate of the slice to be '
              'ignored.'))
    ignore_volumes = traits.List(
        traits.Int,
        argstr='-ignorevolumes %s',
        sep=' ',
        position=2,
        minlen=1,
        desc=('Requires two values (i.e. [2 5 6] for '
              '[Volumes] Ignores the image volumes '
              'specified when computing the tensor.'))
    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc=("Do not display information messages or progress "
              "status."))
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class DWI2TensorOutputSpec(TraitedSpec):
    tensor = File(
        exists=True, desc='path/name of output diffusion tensor image')


class DWI2Tensor(CommandLine):
    """
    Converts diffusion-weighted images to tensor images.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> dwi2tensor = mrt.DWI2Tensor()
    >>> dwi2tensor.inputs.in_file = 'dwi.mif'
    >>> dwi2tensor.inputs.encoding_file = 'encoding.txt'
    >>> dwi2tensor.cmdline
    'dwi2tensor -grad encoding.txt dwi.mif dwi_tensor.mif'
    >>> dwi2tensor.run()                                   # doctest: +SKIP
    """

    _cmd = 'dwi2tensor'
    input_spec = DWI2TensorInputSpec
    output_spec = DWI2TensorOutputSpec


class Tensor2VectorInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Diffusion tensor image')
    out_filename = File(
        genfile=True, argstr='%s', position=-1, desc='Output vector filename')
    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc="Do not display information messages or progress status.")
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class Tensor2VectorOutputSpec(TraitedSpec):
    vector = File(
        exists=True,
        desc=
        'the output image of the major eigenvectors of the diffusion tensor image.'
    )


class Tensor2Vector(CommandLine):
    """
    Generates a map of the major eigenvectors of the tensors in each voxel.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> tensor2vector = mrt.Tensor2Vector()
    >>> tensor2vector.inputs.in_file = 'dwi_tensor.mif'
    >>> tensor2vector.run()                             # doctest: +SKIP
    """

    _cmd = 'tensor2vector'
    input_spec = Tensor2VectorInputSpec
    output_spec = Tensor2VectorOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['vector'] = self.inputs.out_filename
        if not isdefined(outputs['vector']):
            outputs['vector'] = op.abspath(self._gen_outfilename())
        else:
            outputs['vector'] = op.abspath(outputs['vector'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_vector.mif'


class Tensor2FractionalAnisotropyInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Diffusion tensor image')
    out_filename = File(
        genfile=True,
        argstr='%s',
        position=-1,
        desc='Output Fractional Anisotropy filename')
    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc="Do not display information messages or progress status.")
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class Tensor2FractionalAnisotropyOutputSpec(TraitedSpec):
    FA = File(
        exists=True,
        desc=
        'the output image of the major eigenvectors of the diffusion tensor image.'
    )


class Tensor2FractionalAnisotropy(CommandLine):
    """
    Generates a map of the fractional anisotropy in each voxel.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> tensor2FA = mrt.Tensor2FractionalAnisotropy()
    >>> tensor2FA.inputs.in_file = 'dwi_tensor.mif'
    >>> tensor2FA.run()                                 # doctest: +SKIP
    """

    _cmd = 'tensor2FA'
    input_spec = Tensor2FractionalAnisotropyInputSpec
    output_spec = Tensor2FractionalAnisotropyOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['FA'] = self.inputs.out_filename
        if not isdefined(outputs['FA']):
            outputs['FA'] = op.abspath(self._gen_outfilename())
        else:
            outputs['FA'] = op.abspath(outputs['FA'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_FA.mif'


class Tensor2ApparentDiffusionInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Diffusion tensor image')
    out_filename = File(
        genfile=True,
        argstr='%s',
        position=-1,
        desc='Output Fractional Anisotropy filename')
    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc="Do not display information messages or progress status.")
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class Tensor2ApparentDiffusionOutputSpec(TraitedSpec):
    ADC = File(
        exists=True,
        desc=
        'the output image of the major eigenvectors of the diffusion tensor image.'
    )


class Tensor2ApparentDiffusion(CommandLine):
    """
    Generates a map of the apparent diffusion coefficient (ADC) in each voxel

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> tensor2ADC = mrt.Tensor2ApparentDiffusion()
    >>> tensor2ADC.inputs.in_file = 'dwi_tensor.mif'
    >>> tensor2ADC.run()                                # doctest: +SKIP
    """

    _cmd = 'tensor2ADC'
    input_spec = Tensor2ApparentDiffusionInputSpec
    output_spec = Tensor2ApparentDiffusionOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['ADC'] = self.inputs.out_filename
        if not isdefined(outputs['ADC']):
            outputs['ADC'] = op.abspath(self._gen_outfilename())
        else:
            outputs['ADC'] = op.abspath(outputs['ADC'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_ADC.mif'


class MRMultiplyInputSpec(CommandLineInputSpec):
    in_files = InputMultiPath(
        File(exists=True),
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Input images to be multiplied')
    out_filename = File(
        genfile=True, argstr='%s', position=-1, desc='Output image filename')
    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc="Do not display information messages or progress status.")
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class MRMultiplyOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='the output image of the multiplication')


class MRMultiply(CommandLine):
    """
    Multiplies two images.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> MRmult = mrt.MRMultiply()
    >>> MRmult.inputs.in_files = ['dwi.mif', 'dwi_WMProb.mif']
    >>> MRmult.run()                                             # doctest: +SKIP
    """

    _cmd = 'mrmult'
    input_spec = MRMultiplyInputSpec
    output_spec = MRMultiplyOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = self.inputs.out_filename
        if not isdefined(outputs['out_file']):
            outputs['out_file'] = op.abspath(self._gen_outfilename())
        else:
            outputs['out_file'] = op.abspath(outputs['out_file'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_files[0])
        return name + '_MRMult.mif'


class MRTrixViewerInputSpec(CommandLineInputSpec):
    in_files = InputMultiPath(
        File(exists=True),
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Input images to be viewed')
    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc="Do not display information messages or progress status.")
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class MRTrixViewerOutputSpec(TraitedSpec):
    pass


class MRTrixViewer(CommandLine):
    """
    Loads the input images in the MRTrix Viewer.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> MRview = mrt.MRTrixViewer()
    >>> MRview.inputs.in_files = 'dwi.mif'
    >>> MRview.run()                                    # doctest: +SKIP
    """

    _cmd = 'mrview'
    input_spec = MRTrixViewerInputSpec
    output_spec = MRTrixViewerOutputSpec

    def _list_outputs(self):
        return


class MRTrixInfoInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Input images to be read')


class MRTrixInfoOutputSpec(TraitedSpec):
    pass


class MRTrixInfo(CommandLine):
    """
    Prints out relevant header information found in the image specified.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> MRinfo = mrt.MRTrixInfo()
    >>> MRinfo.inputs.in_file = 'dwi.mif'
    >>> MRinfo.run()                                    # doctest: +SKIP
    """

    _cmd = 'mrinfo'
    input_spec = MRTrixInfoInputSpec
    output_spec = MRTrixInfoOutputSpec

    def _list_outputs(self):
        return


class GenerateWhiteMatterMaskInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-3,
        desc='Diffusion-weighted images')
    binary_mask = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Binary brain mask')
    out_WMProb_filename = File(
        genfile=True,
        argstr='%s',
        position=-1,
        desc='Output WM probability image filename')
    encoding_file = File(
        exists=True,
        argstr='-grad %s',
        mandatory=True,
        position=1,
        desc=
        'Gradient encoding, supplied as a 4xN text file with each line is in the format [ X Y Z b ], where [ X Y Z ] describe the direction of the applied gradient, and b gives the b-value in units (1000 s/mm^2). See FSL2MRTrix'
    )
    noise_level_margin = traits.Float(
        argstr='-margin %s',
        desc=
        'Specify the width of the margin on either side of the image to be used to estimate the noise level (default = 10)'
    )


class GenerateWhiteMatterMaskOutputSpec(TraitedSpec):
    WMprobabilitymap = File(exists=True, desc='WMprobabilitymap')


class GenerateWhiteMatterMask(CommandLine):
    """
    Generates a white matter probability mask from the DW images.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> genWM = mrt.GenerateWhiteMatterMask()
    >>> genWM.inputs.in_file = 'dwi.mif'
    >>> genWM.inputs.encoding_file = 'encoding.txt'
    >>> genWM.run()                                     # doctest: +SKIP
    """

    _cmd = 'gen_WM_mask'
    input_spec = GenerateWhiteMatterMaskInputSpec
    output_spec = GenerateWhiteMatterMaskOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['WMprobabilitymap'] = op.abspath(self._gen_outfilename())
        return outputs

    def _gen_filename(self, name):
        if name == 'out_WMProb_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_WMProb.mif'


class ErodeInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Input mask image to be eroded')
    out_filename = File(
        genfile=True, argstr='%s', position=-1, desc='Output image filename')
    number_of_passes = traits.Int(
        argstr='-npass %s', desc='the number of passes (default: 1)')
    dilate = traits.Bool(
        argstr='-dilate',
        position=1,
        desc="Perform dilation rather than erosion")
    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc="Do not display information messages or progress status.")
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class ErodeOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='the output image')


class Erode(CommandLine):
    """
    Erode (or dilates) a mask (i.e. binary) image

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> erode = mrt.Erode()
    >>> erode.inputs.in_file = 'mask.mif'
    >>> erode.run()                                     # doctest: +SKIP
    """
    _cmd = 'erode'
    input_spec = ErodeInputSpec
    output_spec = ErodeOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = self.inputs.out_filename
        if not isdefined(outputs['out_file']):
            outputs['out_file'] = op.abspath(self._gen_outfilename())
        else:
            outputs['out_file'] = op.abspath(outputs['out_file'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_erode.mif'


class ThresholdInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='The input image to be thresholded')
    out_filename = File(
        genfile=True,
        argstr='%s',
        position=-1,
        desc='The output binary image mask.')
    absolute_threshold_value = traits.Float(
        argstr='-abs %s',
        desc='Specify threshold value as absolute intensity.')
    percentage_threshold_value = traits.Float(
        argstr='-percent %s',
        desc=
        'Specify threshold value as a percentage of the peak intensity in the input image.'
    )
    invert = traits.Bool(
        argstr='-invert', position=1, desc="Invert output binary mask")
    replace_zeros_with_NaN = traits.Bool(
        argstr='-nan', position=1, desc="Replace all zero values with NaN")
    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc="Do not display information messages or progress status.")
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class ThresholdOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='The output binary image mask.')


class Threshold(CommandLine):
    """
    Create bitwise image by thresholding image intensity.

    By default, the threshold level is determined using a histogram analysis
    to cut out the background. Otherwise, the threshold intensity can be
    specified using command line options.
    Note that only the first study is used for thresholding.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> thresh = mrt.Threshold()
    >>> thresh.inputs.in_file = 'wm_mask.mif'
    >>> thresh.run()                                             # doctest: +SKIP
    """

    _cmd = 'threshold'
    input_spec = ThresholdInputSpec
    output_spec = ThresholdOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = self.inputs.out_filename
        if not isdefined(outputs['out_file']):
            outputs['out_file'] = op.abspath(self._gen_outfilename())
        else:
            outputs['out_file'] = op.abspath(outputs['out_file'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_thresh.mif'


class MedianFilter3DInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Input images to be smoothed')
    out_filename = File(
        genfile=True, argstr='%s', position=-1, desc='Output image filename')
    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc="Do not display information messages or progress status.")
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class MedianFilter3DOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='the output image')


class MedianFilter3D(CommandLine):
    """
    Smooth images using a 3x3x3 median filter.

    Example
    -------

    >>> import nipype.interfaces.mrtrix as mrt
    >>> median3d = mrt.MedianFilter3D()
    >>> median3d.inputs.in_file = 'mask.mif'
    >>> median3d.run()                                  # doctest: +SKIP
    """

    _cmd = 'median3D'
    input_spec = MedianFilter3DInputSpec
    output_spec = MedianFilter3DOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = self.inputs.out_filename
        if not isdefined(outputs['out_file']):
            outputs['out_file'] = op.abspath(self._gen_outfilename())
        else:
            outputs['out_file'] = op.abspath(outputs['out_file'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_median3D.mif'


class MRTransformInputSpec(CommandLineInputSpec):
    in_files = InputMultiPath(
        File(exists=True),
        argstr='%s',
        mandatory=True,
        position=-2,
        desc='Input images to be transformed')
    out_filename = File(
        genfile=True, argstr='%s', position=-1, desc='Output image')
    invert = traits.Bool(
        argstr='-inverse',
        position=1,
        desc="Invert the specified transform before using it")
    replace_transform = traits.Bool(
        argstr='-replace',
        position=1,
        desc=
        "replace the current transform by that specified, rather than applying it to the current transform"
    )
    transformation_file = File(
        exists=True,
        argstr='-transform %s',
        position=1,
        desc='The transform to apply, in the form of a 4x4 ascii file.')
    template_image = File(
        exists=True,
        argstr='-template %s',
        position=1,
        desc='Reslice the input image to match the specified template image.')
    reference_image = File(
        exists=True,
        argstr='-reference %s',
        position=1,
        desc=
        'in case the transform supplied maps from the input image onto a reference image, use this option to specify the reference. Note that this implicitly sets the -replace option.'
    )
    flip_x = traits.Bool(
        argstr='-flipx',
        position=1,
        desc=
        "assume the transform is supplied assuming a coordinate system with the x-axis reversed relative to the MRtrix convention (i.e. x increases from right to left). This is required to handle transform matrices produced by FSL's FLIRT command. This is only used in conjunction with the -reference option."
    )
    quiet = traits.Bool(
        argstr='-quiet',
        position=1,
        desc="Do not display information messages or progress status.")
    debug = traits.Bool(
        argstr='-debug', position=1, desc="Display debugging messages.")


class MRTransformOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='the output image of the transformation')


class MRTransform(CommandLine):
    """
    Apply spatial transformations or reslice images

    Example
    -------

    >>> MRxform = MRTransform()
    >>> MRxform.inputs.in_files = 'anat_coreg.mif'
    >>> MRxform.run()                                   # doctest: +SKIP
    """

    _cmd = 'mrtransform'
    input_spec = MRTransformInputSpec
    output_spec = MRTransformOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = self.inputs.out_filename
        if not isdefined(outputs['out_file']):
            outputs['out_file'] = op.abspath(self._gen_outfilename())
        else:
            outputs['out_file'] = op.abspath(outputs['out_file'])
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_files[0])
        return name + '_MRTransform.mif'



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (CommandLine, CommandLineInputSpec, SEMLikeCommandLine,
                     TraitedSpec, File, Directory, traits, isdefined,
                     InputMultiPath, OutputMultiPath)


class HistogramMatchingFilterInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="The Input image to be computed for statistics",
        exists=True,
        argstr="--inputVolume %s")
    referenceVolume = File(
        desc="The Input image to be computed for statistics",
        exists=True,
        argstr="--referenceVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output Image File Name",
        argstr="--outputVolume %s")
    referenceBinaryVolume = File(
        desc="referenceBinaryVolume",
        exists=True,
        argstr="--referenceBinaryVolume %s")
    inputBinaryVolume = File(
        desc="inputBinaryVolume", exists=True, argstr="--inputBinaryVolume %s")
    numberOfMatchPoints = traits.Int(
        desc=" number of histogram matching points",
        argstr="--numberOfMatchPoints %d")
    numberOfHistogramBins = traits.Int(
        desc=" number of histogram bin", argstr="--numberOfHistogramBins %d")
    writeHistogram = traits.Str(
        desc=
        " decide if histogram data would be written with prefixe of the file name",
        argstr="--writeHistogram %s")
    histogramAlgorithm = traits.Enum(
        "OtsuHistogramMatching",
        desc=" histogram algrithm selection",
        argstr="--histogramAlgorithm %s")
    verbose = traits.Bool(
        desc=" verbose mode running for debbuging", argstr="--verbose ")


class HistogramMatchingFilterOutputSpec(TraitedSpec):
    outputVolume = File(desc="Output Image File Name", exists=True)


class HistogramMatchingFilter(SEMLikeCommandLine):
    """title: Write Out Image Intensities

category: BRAINS.Utilities

description: For Analysis

version: 0.1

contributor: University of Iowa Department of Psychiatry, http:://www.psychiatry.uiowa.edu

"""

    input_spec = HistogramMatchingFilterInputSpec
    output_spec = HistogramMatchingFilterOutputSpec
    _cmd = " HistogramMatchingFilter "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}
    _redirect_x = False


class GenerateEdgeMapImageInputSpec(CommandLineInputSpec):
    inputMRVolumes = InputMultiPath(
        File(exists=True),
        desc=
        "List of input structural MR volumes to create the maximum edgemap",
        argstr="--inputMRVolumes %s...")
    inputMask = File(
        desc=
        "Input mask file name. If set, image histogram percentiles will be calculated within the mask",
        exists=True,
        argstr="--inputMask %s")
    minimumOutputRange = traits.Int(
        desc=
        "Map lower quantile and below to minimum output range. It should be a small number greater than zero. Default is 1",
        argstr="--minimumOutputRange %d")
    maximumOutputRange = traits.Int(
        desc=
        "Map upper quantile and above to maximum output range. Default is 255 that is the maximum range of unsigned char",
        argstr="--maximumOutputRange %d")
    lowerPercentileMatching = traits.Float(
        desc=
        "Map lower quantile and below to minOutputRange. It should be a value between zero and one",
        argstr="--lowerPercentileMatching %f")
    upperPercentileMatching = traits.Float(
        desc=
        "Map upper quantile and above to maxOutputRange. It should be a value between zero and one",
        argstr="--upperPercentileMatching %f")
    outputEdgeMap = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="output edgemap file name",
        argstr="--outputEdgeMap %s")
    outputMaximumGradientImage = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="output gradient image file name",
        argstr="--outputMaximumGradientImage %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class GenerateEdgeMapImageOutputSpec(TraitedSpec):
    outputEdgeMap = File(desc="(required) output file name", exists=True)
    outputMaximumGradientImage = File(
        desc="output gradient image file name", exists=True)


class GenerateEdgeMapImage(SEMLikeCommandLine):
    """title: GenerateEdgeMapImage

category: BRAINS.Utilities

description: Automatic edgemap generation for edge-guided super-resolution reconstruction

version: 1.0

contributor: Ali Ghayoor

"""

    input_spec = GenerateEdgeMapImageInputSpec
    output_spec = GenerateEdgeMapImageOutputSpec
    _cmd = " GenerateEdgeMapImage "
    _outputs_filenames = {
        'outputEdgeMap': 'outputEdgeMap',
        'outputMaximumGradientImage': 'outputMaximumGradientImage'
    }
    _redirect_x = False


class GeneratePurePlugMaskInputSpec(CommandLineInputSpec):
    inputImageModalities = InputMultiPath(
        File(exists=True),
        desc="List of input image file names to create pure plugs mask",
        argstr="--inputImageModalities %s...")
    threshold = traits.Float(
        desc="threshold value to define class membership",
        argstr="--threshold %f")
    numberOfSubSamples = InputMultiPath(
        traits.Int,
        desc=
        "Number of continous index samples taken at each direction of lattice space for each plug volume",
        sep=",",
        argstr="--numberOfSubSamples %s")
    outputMaskFile = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output binary mask file name",
        argstr="--outputMaskFile %s")


class GeneratePurePlugMaskOutputSpec(TraitedSpec):
    outputMaskFile = File(
        desc="(required) Output binary mask file name", exists=True)


class GeneratePurePlugMask(SEMLikeCommandLine):
    """title: GeneratePurePlugMask

category: BRAINS.Utilities

description: This program gets several modality image files and returns a binary mask that defines the pure plugs

version: 1.0

contributor: Ali Ghayoor

"""

    input_spec = GeneratePurePlugMaskInputSpec
    output_spec = GeneratePurePlugMaskOutputSpec
    _cmd = " GeneratePurePlugMask "
    _outputs_filenames = {'outputMaskFile': 'outputMaskFile'}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import RealignUnwarp


def test_RealignUnwarp_inputs():
    input_map = dict(
        est_basis_func=dict(field='uweoptions.basfcn', ),
        est_first_order_effects=dict(field='uweoptions.fot', ),
        est_jacobian_deformations=dict(field='uweoptions.jm', ),
        est_num_of_iterations=dict(
            field='uweoptions.noi',
            maxlen=1,
            minlen=1,
            usedefault=True,
        ),
        est_re_est_mov_par=dict(field='uweoptions.rem', ),
        est_reg_factor=dict(
            field='uweoptions.lambda',
            maxlen=1,
            minlen=1,
            usedefault=True,
        ),
        est_reg_order=dict(field='uweoptions.regorder', ),
        est_second_order_effects=dict(field='uweoptions.sot', ),
        est_taylor_expansion_point=dict(
            field='uweoptions.expround',
            usedefault=True,
        ),
        est_unwarp_fwhm=dict(field='uweoptions.uwfwhm', ),
        fwhm=dict(field='eoptions.fwhm', ),
        in_files=dict(
            copyfile=True,
            field='data.scans',
            mandatory=True,
        ),
        interp=dict(field='eoptions.einterp', ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        out_prefix=dict(
            field='uwroptions.prefix',
            usedefault=True,
        ),
        paths=dict(),
        phase_map=dict(
            copyfile=False,
            extensions=None,
            field='data.pmscan',
        ),
        quality=dict(field='eoptions.quality', ),
        register_to_mean=dict(field='eoptions.rtm', ),
        reslice_interp=dict(field='uwroptions.rinterp', ),
        reslice_mask=dict(field='uwroptions.mask', ),
        reslice_which=dict(
            field='uwroptions.uwwhich',
            maxlen=2,
            minlen=2,
            usedefault=True,
        ),
        reslice_wrap=dict(field='uwroptions.wrap', ),
        separation=dict(field='eoptions.sep', ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
        weight_img=dict(
            extensions=None,
            field='eoptions.weight',
        ),
        wrap=dict(field='eoptions.ewrap', ),
    )
    inputs = RealignUnwarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RealignUnwarp_outputs():
    output_map = dict(
        mean_image=dict(extensions=None, ),
        modified_in_files=dict(),
        realigned_unwarped_files=dict(),
        realignment_parameters=dict(),
    )
    outputs = RealignUnwarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import AlignEpiAnatPy


def test_AlignEpiAnatPy_inputs():
    input_map = dict(
        anat=dict(
            argstr='-anat %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        anat2epi=dict(argstr='-anat2epi', ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epi2anat=dict(argstr='-epi2anat', ),
        epi_base=dict(
            argstr='-epi_base %s',
            mandatory=True,
        ),
        epi_strip=dict(argstr='-epi_strip %s', ),
        in_file=dict(
            argstr='-epi %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        outputtype=dict(),
        py27_path=dict(usedefault=True, ),
        save_skullstrip=dict(argstr='-save_skullstrip', ),
        suffix=dict(
            argstr='-suffix %s',
            usedefault=True,
        ),
        tshift=dict(
            argstr='-tshift %s',
            usedefault=True,
        ),
        volreg=dict(
            argstr='-volreg %s',
            usedefault=True,
        ),
    )
    inputs = AlignEpiAnatPy.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_AlignEpiAnatPy_outputs():
    output_map = dict(
        anat_al_mat=dict(extensions=None, ),
        anat_al_orig=dict(extensions=None, ),
        epi_al_mat=dict(extensions=None, ),
        epi_al_orig=dict(extensions=None, ),
        epi_al_tlrc_mat=dict(extensions=None, ),
        epi_reg_al_mat=dict(extensions=None, ),
        epi_tlrc_al=dict(extensions=None, ),
        epi_vr_al_mat=dict(extensions=None, ),
        epi_vr_motion=dict(extensions=None, ),
        skullstrip=dict(extensions=None, ),
    )
    outputs = AlignEpiAnatPy.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (CommandLine, CommandLineInputSpec, SEMLikeCommandLine,
                     TraitedSpec, File, Directory, traits, isdefined,
                     InputMultiPath, OutputMultiPath)


class BRAINSResizeInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Image To Scale", exists=True, argstr="--inputVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Resulting scaled image",
        argstr="--outputVolume %s")
    pixelType = traits.Enum(
        "float",
        "short",
        "ushort",
        "int",
        "uint",
        "uchar",
        "binary",
        desc=
        "Specifies the pixel type for the input/output images.  The \'binary\' pixel type uses a modified algorithm whereby the image is read in as unsigned char, a signed distance map is created, signed distance map is resampled, and then a thresholded image of type unsigned char is written to disk.",
        argstr="--pixelType %s")
    scaleFactor = traits.Float(
        desc="The scale factor for the image spacing.",
        argstr="--scaleFactor %f")


class BRAINSResizeOutputSpec(TraitedSpec):
    outputVolume = File(desc="Resulting scaled image", exists=True)


class BRAINSResize(SEMLikeCommandLine):
    """title: Resize Image (BRAINS)

category: Registration

description: This program is useful for downsampling an image by a constant scale factor.

version: 3.0.0

license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

contributor: This tool was developed by Hans Johnson.

acknowledgements: The development of this tool was supported by funding from grants NS050568 and NS40068 from the National Institute of Neurological Disorders and Stroke and grants MH31593, MH40856, from the National Institute of Mental Health.

"""

    input_spec = BRAINSResizeInputSpec
    output_spec = BRAINSResizeOutputSpec
    _cmd = " BRAINSResize "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utilities import GenerateEdgeMapImage


def test_GenerateEdgeMapImage_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputMRVolumes=dict(argstr='--inputMRVolumes %s...', ),
        inputMask=dict(
            argstr='--inputMask %s',
            extensions=None,
        ),
        lowerPercentileMatching=dict(argstr='--lowerPercentileMatching %f', ),
        maximumOutputRange=dict(argstr='--maximumOutputRange %d', ),
        minimumOutputRange=dict(argstr='--minimumOutputRange %d', ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        outputEdgeMap=dict(
            argstr='--outputEdgeMap %s',
            hash_files=False,
        ),
        outputMaximumGradientImage=dict(
            argstr='--outputMaximumGradientImage %s',
            hash_files=False,
        ),
        upperPercentileMatching=dict(argstr='--upperPercentileMatching %f', ),
    )
    inputs = GenerateEdgeMapImage.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_GenerateEdgeMapImage_outputs():
    output_map = dict(
        outputEdgeMap=dict(extensions=None, ),
        outputMaximumGradientImage=dict(extensions=None, ),
    )
    outputs = GenerateEdgeMapImage.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..visualization import CreateTiledMosaic


def test_CreateTiledMosaic_inputs():
    input_map = dict(
        alpha_value=dict(argstr='-a %.2f', ),
        args=dict(argstr='%s', ),
        direction=dict(argstr='-d %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        flip_slice=dict(argstr='-f %s', ),
        input_image=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        mask_image=dict(
            argstr='-x %s',
            extensions=None,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        output_image=dict(
            argstr='-o %s',
            usedefault=True,
        ),
        pad_or_crop=dict(argstr='-p %s', ),
        permute_axes=dict(argstr='-g', ),
        rgb_image=dict(
            argstr='-r %s',
            extensions=None,
            mandatory=True,
        ),
        slices=dict(argstr='-s %s', ),
        tile_geometry=dict(argstr='-t %s', ),
    )
    inputs = CreateTiledMosaic.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_CreateTiledMosaic_outputs():
    output_map = dict(output_image=dict(extensions=None, ), )
    outputs = CreateTiledMosaic.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..segmentation import N4BiasFieldCorrection


def test_N4BiasFieldCorrection_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bias_image=dict(
            extensions=None,
            hash_files=False,
        ),
        bspline_fitting_distance=dict(argstr='--bspline-fitting %s', ),
        bspline_order=dict(requires=['bspline_fitting_distance'], ),
        convergence_threshold=dict(requires=['n_iterations'], ),
        copy_header=dict(
            mandatory=True,
            usedefault=True,
        ),
        dimension=dict(
            argstr='-d %d',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        histogram_sharpening=dict(
            argstr='--histogram-sharpening [%g,%g,%d]', ),
        input_image=dict(
            argstr='--input-image %s',
            extensions=None,
            mandatory=True,
        ),
        mask_image=dict(
            argstr='--mask-image %s',
            extensions=None,
        ),
        n_iterations=dict(argstr='--convergence %s', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        output_image=dict(
            argstr='--output %s',
            hash_files=False,
            keep_extension=True,
            name_source=['input_image'],
            name_template='%s_corrected',
        ),
        rescale_intensities=dict(
            argstr='-r',
            min_ver='2.1.0',
            usedefault=True,
        ),
        save_bias=dict(
            mandatory=True,
            usedefault=True,
            xor=['bias_image'],
        ),
        shrink_factor=dict(argstr='--shrink-factor %d', ),
        weight_image=dict(
            argstr='--weight-image %s',
            extensions=None,
        ),
    )
    inputs = N4BiasFieldCorrection.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_N4BiasFieldCorrection_outputs():
    output_map = dict(
        bias_image=dict(extensions=None, ),
        output_image=dict(extensions=None, ),
    )
    outputs = N4BiasFieldCorrection.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..minc import Convert


def test_Convert_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        chunk=dict(argstr='-chunk %d', ),
        clobber=dict(
            argstr='-clobber',
            usedefault=True,
        ),
        compression=dict(argstr='-compress %s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        input_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        output_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            hash_files=False,
            name_source=['input_file'],
            name_template='%s_convert_output.mnc',
            position=-1,
        ),
        template=dict(argstr='-template', ),
        two=dict(argstr='-2', ),
    )
    inputs = Convert.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Convert_outputs():
    output_map = dict(output_file=dict(extensions=None, ), )
    outputs = Convert.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..gtract import gtractFastMarchingTracking


def test_gtractFastMarchingTracking_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        costStepSize=dict(argstr='--costStepSize %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputAnisotropyVolume=dict(
            argstr='--inputAnisotropyVolume %s',
            extensions=None,
        ),
        inputCostVolume=dict(
            argstr='--inputCostVolume %s',
            extensions=None,
        ),
        inputStartingSeedsLabelMapVolume=dict(
            argstr='--inputStartingSeedsLabelMapVolume %s',
            extensions=None,
        ),
        inputTensorVolume=dict(
            argstr='--inputTensorVolume %s',
            extensions=None,
        ),
        maximumStepSize=dict(argstr='--maximumStepSize %f', ),
        minimumStepSize=dict(argstr='--minimumStepSize %f', ),
        numberOfIterations=dict(argstr='--numberOfIterations %d', ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        outputTract=dict(
            argstr='--outputTract %s',
            hash_files=False,
        ),
        seedThreshold=dict(argstr='--seedThreshold %f', ),
        startingSeedsLabel=dict(argstr='--startingSeedsLabel %d', ),
        trackingThreshold=dict(argstr='--trackingThreshold %f', ),
        writeXMLPolyDataFile=dict(argstr='--writeXMLPolyDataFile ', ),
    )
    inputs = gtractFastMarchingTracking.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_gtractFastMarchingTracking_outputs():
    output_map = dict(outputTract=dict(extensions=None, ), )
    outputs = gtractFastMarchingTracking.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import DicomImport


def test_DicomImport_inputs():
    input_map = dict(
        format=dict(
            field='convopts.format',
            usedefault=True,
        ),
        icedims=dict(
            field='convopts.icedims',
            usedefault=True,
        ),
        in_files=dict(
            field='data',
            mandatory=True,
        ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        output_dir=dict(
            field='outdir',
            usedefault=True,
        ),
        output_dir_struct=dict(
            field='root',
            usedefault=True,
        ),
        paths=dict(),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
    )
    inputs = DicomImport.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DicomImport_outputs():
    output_map = dict(out_files=dict(), )
    outputs = DicomImport.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..registration import CompositeTransformUtil


def test_CompositeTransformUtil_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s...',
            mandatory=True,
            position=3,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            position=2,
        ),
        output_prefix=dict(
            argstr='%s',
            position=4,
            usedefault=True,
        ),
        process=dict(
            argstr='--%s',
            position=1,
            usedefault=True,
        ),
    )
    inputs = CompositeTransformUtil.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_CompositeTransformUtil_outputs():
    output_map = dict(
        affine_transform=dict(extensions=None, ),
        displacement_field=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
    )
    outputs = CompositeTransformUtil.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import ExtractROI


def test_ExtractROI_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        crop_list=dict(
            argstr='%s',
            position=2,
            xor=[
                'x_min', 'x_size', 'y_min', 'y_size', 'z_min', 'z_size',
                't_min', 't_size'
            ],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        output_type=dict(),
        roi_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            hash_files=False,
            position=1,
        ),
        t_min=dict(
            argstr='%d',
            position=8,
        ),
        t_size=dict(
            argstr='%d',
            position=9,
        ),
        x_min=dict(
            argstr='%d',
            position=2,
        ),
        x_size=dict(
            argstr='%d',
            position=3,
        ),
        y_min=dict(
            argstr='%d',
            position=4,
        ),
        y_size=dict(
            argstr='%d',
            position=5,
        ),
        z_min=dict(
            argstr='%d',
            position=6,
        ),
        z_size=dict(
            argstr='%d',
            position=7,
        ),
    )
    inputs = ExtractROI.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ExtractROI_outputs():
    output_map = dict(roi_file=dict(extensions=None, ), )
    outputs = ExtractROI.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Qwarp


def test_Qwarp_inputs():
    input_map = dict(
        Qfinal=dict(argstr='-Qfinal', ),
        Qonly=dict(argstr='-Qonly', ),
        allineate=dict(argstr='-allineate', ),
        allineate_opts=dict(
            argstr='-allineate_opts %s',
            requires=['allineate'],
        ),
        allsave=dict(
            argstr='-allsave',
            xor=['nopadWARP', 'duplo', 'plusminus'],
        ),
        args=dict(argstr='%s', ),
        ballopt=dict(
            argstr='-ballopt',
            xor=['workhard', 'boxopt'],
        ),
        base_file=dict(
            argstr='-base %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        baxopt=dict(
            argstr='-boxopt',
            xor=['workhard', 'ballopt'],
        ),
        blur=dict(argstr='-blur %s', ),
        duplo=dict(
            argstr='-duplo',
            xor=[
                'gridlist', 'maxlev', 'inilev', 'iniwarp', 'plusminus',
                'allsave'
            ],
        ),
        emask=dict(
            argstr='-emask %s',
            copyfile=False,
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        expad=dict(
            argstr='-expad %d',
            xor=['nopadWARP'],
        ),
        gridlist=dict(
            argstr='-gridlist %s',
            copyfile=False,
            extensions=None,
            xor=['duplo', 'plusminus'],
        ),
        hel=dict(
            argstr='-hel',
            xor=['nmi', 'mi', 'lpc', 'lpa', 'pear'],
        ),
        in_file=dict(
            argstr='-source %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        inilev=dict(
            argstr='-inilev %d',
            xor=['duplo'],
        ),
        iniwarp=dict(
            argstr='-iniwarp %s',
            xor=['duplo'],
        ),
        iwarp=dict(
            argstr='-iwarp',
            xor=['plusminus'],
        ),
        lpa=dict(
            argstr='-lpa',
            xor=['nmi', 'mi', 'lpc', 'hel', 'pear'],
        ),
        lpc=dict(
            argstr='-lpc',
            position=-2,
            xor=['nmi', 'mi', 'hel', 'lpa', 'pear'],
        ),
        maxlev=dict(
            argstr='-maxlev %d',
            position=-1,
            xor=['duplo'],
        ),
        mi=dict(
            argstr='-mi',
            xor=['mi', 'hel', 'lpc', 'lpa', 'pear'],
        ),
        minpatch=dict(argstr='-minpatch %d', ),
        nmi=dict(
            argstr='-nmi',
            xor=['nmi', 'hel', 'lpc', 'lpa', 'pear'],
        ),
        noXdis=dict(argstr='-noXdis', ),
        noYdis=dict(argstr='-noYdis', ),
        noZdis=dict(argstr='-noZdis', ),
        noneg=dict(argstr='-noneg', ),
        nopad=dict(argstr='-nopad', ),
        nopadWARP=dict(
            argstr='-nopadWARP',
            xor=['allsave', 'expad'],
        ),
        nopenalty=dict(argstr='-nopenalty', ),
        nowarp=dict(argstr='-nowarp', ),
        noweight=dict(argstr='-noweight', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            name_source=['in_file'],
            name_template='ppp_%s',
        ),
        out_weight_file=dict(
            argstr='-wtprefix %s',
            extensions=None,
        ),
        outputtype=dict(),
        overwrite=dict(argstr='-overwrite', ),
        pblur=dict(argstr='-pblur %s', ),
        pear=dict(argstr='-pear', ),
        penfac=dict(argstr='-penfac %f', ),
        plusminus=dict(
            argstr='-plusminus',
            xor=['duplo', 'allsave', 'iwarp'],
        ),
        quiet=dict(
            argstr='-quiet',
            xor=['verb'],
        ),
        resample=dict(argstr='-resample', ),
        verb=dict(
            argstr='-verb',
            xor=['quiet'],
        ),
        wball=dict(argstr='-wball %s', ),
        weight=dict(
            argstr='-weight %s',
            extensions=None,
        ),
        wmask=dict(argstr='-wpass %s %f', ),
        workhard=dict(
            argstr='-workhard',
            xor=['boxopt', 'ballopt'],
        ),
    )
    inputs = Qwarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Qwarp_outputs():
    output_map = dict(
        base_warp=dict(extensions=None, ),
        source_warp=dict(extensions=None, ),
        warped_base=dict(extensions=None, ),
        warped_source=dict(extensions=None, ),
        weights=dict(extensions=None, ),
    )
    outputs = Qwarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..developer import JistCortexSurfaceMeshInflation


def test_JistCortexSurfaceMeshInflation_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inLevelset=dict(
            argstr='--inLevelset %s',
            extensions=None,
        ),
        inLorentzian=dict(argstr='--inLorentzian %s', ),
        inMax=dict(argstr='--inMax %d', ),
        inMean=dict(argstr='--inMean %f', ),
        inSOR=dict(argstr='--inSOR %f', ),
        inStep=dict(argstr='--inStep %d', ),
        inTopology=dict(argstr='--inTopology %s', ),
        null=dict(argstr='--null %s', ),
        outInflated=dict(
            argstr='--outInflated %s',
            hash_files=False,
        ),
        outOriginal=dict(
            argstr='--outOriginal %s',
            hash_files=False,
        ),
        xDefaultMem=dict(argstr='-xDefaultMem %d', ),
        xMaxProcess=dict(
            argstr='-xMaxProcess %d',
            usedefault=True,
        ),
        xPrefExt=dict(argstr='--xPrefExt %s', ),
    )
    inputs = JistCortexSurfaceMeshInflation.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_JistCortexSurfaceMeshInflation_outputs():
    output_map = dict(
        outInflated=dict(extensions=None, ),
        outOriginal=dict(extensions=None, ),
    )
    outputs = JistCortexSurfaceMeshInflation.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .brains import (
    BRAINSConstellationModeler, landmarksConstellationWeights,
    BRAINSTrimForegroundInDirection, BRAINSLmkTransform, BRAINSMush,
    BRAINSTransformConvert, landmarksConstellationAligner, BRAINSEyeDetector,
    BRAINSLinearModelerEPCA, BRAINSInitializedControlPoints,
    CleanUpOverlapLabels, BRAINSClipInferior,
    GenerateLabelMapFromProbabilityMap, BRAINSAlignMSP,
    BRAINSLandmarkInitializer, insertMidACPCpoint, BRAINSSnapShotWriter,
    JointHistogram, ShuffleVectorsModule, ImageRegionPlotter)



# -*- coding: utf-8 -*-
"""Parallel workflow execution via Condor DAGMan
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open

import os
import sys
import uuid
import time
from warnings import warn

from .base import GraphPluginBase, logger
from ...interfaces.base import CommandLine


class CondorDAGManPlugin(GraphPluginBase):
    """Execute using Condor DAGMan

    The plugin_args input to run can be used to control the DAGMan execution.
    The value of most arguments can be a literal string or a filename, where in
    the latter case the content of the file will be used as the argument value.

    Currently supported options are:

    - submit_template : submit spec template for individual jobs in a DAG (see
                 CondorDAGManPlugin.default_submit_template for the default.
    - initial_specs : additional submit specs that are prepended to any job's
                 submit file
    - override_specs : additional submit specs that are appended to any job's
                 submit file
    - wrapper_cmd : path to an executable that will be started instead of a node
                 script. This is useful for wrapper script that execute certain
                 functionality prior or after a node runs. If this option is
                 given the wrapper command is called with the respective Python
                 executable and the path to the node script as final arguments
    - wrapper_args : optional additional arguments to a wrapper command
    - dagman_args : arguments to be prepended to the arguments of the
                    condor_submit_dag call
    - block : if True the plugin call will block until Condor has finished
                 processing the entire workflow (default: False)
    """

    default_submit_template = """
universe = vanilla
notification = Never
executable = %(executable)s
arguments = %(nodescript)s
output = %(basename)s.out
error = %(basename)s.err
log = %(basename)s.log
getenv = True
"""

    def _get_str_or_file(self, arg):
        if os.path.isfile(arg):
            with open(arg) as f:
                content = f.read()
        else:
            content = arg
        return content

    # XXX feature wishlist
    # - infer data file dependencies from jobs
    # - infer CPU requirements from jobs
    # - infer memory requirements from jobs
    # - looks like right now all jobs come in here, regardless of whether they
    #   actually have to run. would be good to be able to decide whether they
    #   actually have to be scheduled (i.e. output already exist).
    def __init__(self, **kwargs):
        for var, id_, val in \
            (('_template', 'submit_template', self.default_submit_template),
             ('_initial_specs', 'template', ''),
             ('_initial_specs', 'initial_specs', ''),
             ('_override_specs', 'submit_specs', ''),
             ('_override_specs', 'override_specs', ''),
             ('_wrapper_cmd', 'wrapper_cmd', None),
             ('_wrapper_args', 'wrapper_args', ''),
             ('_block', 'block', False),
             ('_dagman_args', 'dagman_args', '')):
            if 'plugin_args' in kwargs \
                    and not kwargs['plugin_args'] is None \
                    and id_ in kwargs['plugin_args']:
                if id_ == 'wrapper_cmd':
                    val = os.path.abspath(kwargs['plugin_args'][id_])
                elif id_ == 'block':
                    val = kwargs['plugin_args'][id_]
                else:
                    val = self._get_str_or_file(kwargs['plugin_args'][id_])
            setattr(self, var, val)
        # TODO remove after some time
        if 'plugin_args' in kwargs \
                and not kwargs['plugin_args'] is None:
            plugin_args = kwargs['plugin_args']
            if 'template' in plugin_args:
                warn(
                    "the 'template' argument is deprecated, use 'initial_specs' instead"
                )
            if 'submit_specs' in plugin_args:
                warn(
                    "the 'submit_specs' argument is deprecated, use 'override_specs' instead"
                )
        super(CondorDAGManPlugin, self).__init__(**kwargs)

    def _submit_graph(self, pyfiles, dependencies, nodes):
        # location of all scripts, place dagman output in here too
        batch_dir, _ = os.path.split(pyfiles[0])
        # DAG description filename
        dagfilename = os.path.join(batch_dir, 'workflow-%s.dag' % uuid.uuid4())
        with open(dagfilename, 'wt') as dagfileptr:
            # loop over all scripts, create submit files, and define them
            # as jobs in the DAG
            for idx, pyscript in enumerate(pyfiles):
                node = nodes[idx]
                # XXX redundant with previous value? or could it change between
                # scripts?
                template, initial_specs, override_specs, wrapper_cmd, wrapper_args = \
                    self._get_args(node,
                                   ["template", "initial_specs",
                                    "override_specs", "wrapper_cmd",
                                    "wrapper_args"])
                # add required slots to the template
                template = '%s\n%s\n%s\nqueue\n' % ('%(initial_specs)s',
                                                    template,
                                                    '%(override_specs)s')
                batch_dir, name = os.path.split(pyscript)
                name = '.'.join(name.split('.')[:-1])
                specs = dict(
                    # TODO make parameter for this,
                    initial_specs=initial_specs,
                    executable=sys.executable,
                    nodescript=pyscript,
                    basename=os.path.join(batch_dir, name),
                    override_specs=override_specs)
                if wrapper_cmd is not None:
                    specs['executable'] = wrapper_cmd
                    specs['nodescript'] = \
                        '%s %s %s' % (wrapper_args % specs,  # give access to variables
                                      sys.executable,
                                      pyscript)
                submitspec = template % specs
                # write submit spec for this job
                submitfile = os.path.join(batch_dir, '%s.submit' % name)
                with open(submitfile, 'wt') as submitfileprt:
                    submitfileprt.writelines(submitspec)
                    submitfileprt.close()
                # define job in DAG
                dagfileptr.write('JOB %i %s\n' % (idx, submitfile))
            # define dependencies in DAG
            for child in dependencies:
                parents = dependencies[child]
                if len(parents):
                    dagfileptr.write('PARENT %s CHILD %i\n' %
                                     (' '.join([str(i) for i in parents]),
                                      child))
        # hand over DAG to condor_dagman
        cmd = CommandLine(
            'condor_submit_dag',
            environ=dict(os.environ),
            resource_monitor=False,
            terminal_output='allatonce')
        # needs -update_submit or re-running a workflow will fail
        cmd.inputs.args = '%s -update_submit %s' % (self._dagman_args,
                                                    dagfilename)
        cmd.run()
        logger.info('submitted all jobs to Condor DAGMan')
        if self._block:
            # wait for DAGMan to settle down, no time wasted it is already running
            time.sleep(10)
            if not os.path.exists('%s.condor.sub' % dagfilename):
                raise EnvironmentError(
                    "DAGMan did not create its submit file, please check the logs"
                )
            # wait for completion
            logger.info('waiting for DAGMan to finish')
            lockfilename = '%s.lock' % dagfilename
            while os.path.exists(lockfilename):
                time.sleep(5)



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import CommandLine, CommandLineInputSpec, SEMLikeCommandLine, TraitedSpec, File, Directory, traits, isdefined, InputMultiPath, OutputMultiPath
import os


class BSplineToDeformationFieldInputSpec(CommandLineInputSpec):
    tfm = File(exists=True, argstr="--tfm %s")
    refImage = File(exists=True, argstr="--refImage %s")
    defImage = traits.Either(
        traits.Bool, File(), hash_files=False, argstr="--defImage %s")


class BSplineToDeformationFieldOutputSpec(TraitedSpec):
    defImage = File(exists=True)


class BSplineToDeformationField(SEMLikeCommandLine):
    """title: BSpline to deformation field

category: Legacy.Converters

description: Create a dense deformation field from a bspline+bulk transform.

version: 0.1.0.$Revision: 2104 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/BSplineToDeformationField

contributor: Andrey Fedorov (SPL, BWH)

acknowledgements: This work is funded by NIH grants R01 CA111288 and U01 CA151261.

"""

    input_spec = BSplineToDeformationFieldInputSpec
    output_spec = BSplineToDeformationFieldOutputSpec
    _cmd = "BSplineToDeformationField "
    _outputs_filenames = {'defImage': 'defImage.nii'}



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import SurfaceSmooth


def test_SurfaceSmooth_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        cortex=dict(
            argstr='--cortex',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fwhm=dict(
            argstr='--fwhm %.4f',
            xor=['smooth_iters'],
        ),
        hemi=dict(
            argstr='--hemi %s',
            mandatory=True,
        ),
        in_file=dict(
            argstr='--sval %s',
            extensions=None,
            mandatory=True,
        ),
        out_file=dict(
            argstr='--tval %s',
            extensions=None,
            genfile=True,
        ),
        reshape=dict(argstr='--reshape', ),
        smooth_iters=dict(
            argstr='--smooth %d',
            xor=['fwhm'],
        ),
        subject_id=dict(
            argstr='--s %s',
            mandatory=True,
        ),
        subjects_dir=dict(),
    )
    inputs = SurfaceSmooth.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SurfaceSmooth_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = SurfaceSmooth.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ....base import (CommandLine, CommandLineInputSpec, SEMLikeCommandLine,
                      TraitedSpec, File, Directory, traits, isdefined,
                      InputMultiPath, OutputMultiPath)


class fibertrackInputSpec(CommandLineInputSpec):
    input_tensor_file = File(
        desc="Tensor Image", exists=True, argstr="--input_tensor_file %s")
    input_roi_file = File(
        desc=
        "The filename of the image which contains the labels used for seeding and constraining the algorithm.",
        exists=True,
        argstr="--input_roi_file %s")
    output_fiber_file = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "The filename for the fiber file produced by the algorithm. This file must end in a .fib or .vtk extension for ITK spatial object and vtkPolyData formats respectively.",
        argstr="--output_fiber_file %s")
    source_label = traits.Int(
        desc=
        "The label of voxels in the labelfile to use for seeding tractography. One tract is seeded from the center of each voxel with this label",
        argstr="--source_label %d")
    target_label = traits.Int(
        desc=
        "The label of voxels in the labelfile used to constrain tractography. Tracts that do not pass through a voxel with this label are rejected. Set this keep all tracts.",
        argstr="--target_label %d")
    forbidden_label = traits.Int(
        desc="Forbidden label", argstr="--forbidden_label %d")
    whole_brain = traits.Bool(
        desc=
        "If this option is enabled all voxels in the image are used to seed tractography. When this option is enabled both source and target labels function as target labels",
        argstr="--whole_brain ")
    max_angle = traits.Float(
        desc="Maximum angle of change in radians", argstr="--max_angle %f")
    step_size = traits.Float(
        desc="Step size in mm for the tracking algorithm",
        argstr="--step_size %f")
    min_fa = traits.Float(
        desc="The minimum FA threshold to continue tractography",
        argstr="--min_fa %f")
    force = traits.Bool(desc="Ignore sanity checks.", argstr="--force ")
    verbose = traits.Bool(desc="produce verbose output", argstr="--verbose ")
    really_verbose = traits.Bool(
        desc="Follow detail of fiber tracking algorithm",
        argstr="--really_verbose ")


class fibertrackOutputSpec(TraitedSpec):
    output_fiber_file = File(
        desc=
        "The filename for the fiber file produced by the algorithm. This file must end in a .fib or .vtk extension for ITK spatial object and vtkPolyData formats respectively.",
        exists=True)


class fibertrack(SEMLikeCommandLine):
    """title: FiberTrack (DTIProcess)

category: Diffusion.Tractography

description: This program implements a simple streamline tractography method based on the principal eigenvector of the tensor field. A fourth order Runge-Kutta integration rule used to advance the streamlines.
As a first parameter you have to input the tensor field (with the --input_tensor_file option). Then the region of interest image file is set with the --input_roi_file. Next you want to set the output fiber file name after the --output_fiber_file option.
You can specify the label value in the input_roi_file with the --target_label, --source_label and  --fobidden_label options. By default target label is 1, source label is 2 and forbidden label is 0. The source label is where the streamlines are seeded, the target label defines the voxels through which the fibers must pass by to be kept in the final fiber file and the forbidden label defines the voxels where the streamlines are stopped if they pass through it. There is also a --whole_brain option which, if enabled, consider both target and source labels of the roi image as target labels and all the voxels of the image are considered as sources.
During the tractography, the --fa_min parameter is used as the minimum value needed at different voxel for the tracking to keep going along a streamline. The --step_size parameter is used for each iteration of the tracking algorithm and defines the length of each step. The --max_angle option defines the maximum angle allowed between two successive segments along the tracked fiber.

version: 1.1.0

documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/Nightly/Extensions/DTIProcess

license: Copyright (c)  Casey Goodlett. All rights reserved.
  See http://www.ia.unc.edu/dev/Copyright.htm for details.
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

contributor: Casey Goodlett

acknowledgements: Hans Johnson(1,3,4); Kent Williams(1); (1=University of Iowa Department of Psychiatry, 3=University of Iowa Department of Biomedical Engineering, 4=University of Iowa Department of Electrical and Computer Engineering) provided conversions to make DTIProcess compatible with Slicer execution, and simplified the stand-alone build requirements by removing the dependancies on boost and a fortran compiler.

"""

    input_spec = fibertrackInputSpec
    output_spec = fibertrackOutputSpec
    _cmd = " fibertrack "
    _outputs_filenames = {'output_fiber_file': 'output_fiber_file.vtk'}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import MRIsConvert


def test_MRIsConvert_inputs():
    input_map = dict(
        annot_file=dict(
            argstr='--annot %s',
            extensions=None,
        ),
        args=dict(argstr='%s', ),
        dataarray_num=dict(argstr='--da_num %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        functional_file=dict(
            argstr='-f %s',
            extensions=None,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        label_file=dict(
            argstr='--label %s',
            extensions=None,
        ),
        labelstats_outfile=dict(
            argstr='--labelstats %s',
            extensions=None,
        ),
        normal=dict(argstr='-n', ),
        origname=dict(argstr='-o %s', ),
        out_datatype=dict(
            mandatory=True,
            xor=['out_file'],
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            mandatory=True,
            position=-1,
            xor=['out_datatype'],
        ),
        parcstats_file=dict(
            argstr='--parcstats %s',
            extensions=None,
        ),
        patch=dict(argstr='-p', ),
        rescale=dict(argstr='-r', ),
        scalarcurv_file=dict(
            argstr='-c %s',
            extensions=None,
        ),
        scale=dict(argstr='-s %.3f', ),
        subjects_dir=dict(),
        talairachxfm_subjid=dict(argstr='-t %s', ),
        to_scanner=dict(argstr='--to-scanner', ),
        to_tkr=dict(argstr='--to-tkr', ),
        vertex=dict(argstr='-v', ),
        xyz_ascii=dict(argstr='-a', ),
    )
    inputs = MRIsConvert.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MRIsConvert_outputs():
    output_map = dict(converted=dict(extensions=None, ), )
    outputs = MRIsConvert.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..longitudinal import RobustTemplate


def test_RobustTemplate_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        auto_detect_sensitivity=dict(
            argstr='--satit',
            mandatory=True,
            xor=['outlier_sensitivity'],
        ),
        average_metric=dict(argstr='--average %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_timepoint=dict(argstr='--fixtp', ),
        in_files=dict(
            argstr='--mov %s',
            mandatory=True,
        ),
        in_intensity_scales=dict(argstr='--iscalein %s', ),
        initial_timepoint=dict(argstr='--inittp %d', ),
        initial_transforms=dict(argstr='--ixforms %s', ),
        intensity_scaling=dict(argstr='--iscale', ),
        no_iteration=dict(argstr='--noit', ),
        num_threads=dict(),
        out_file=dict(
            argstr='--template %s',
            extensions=None,
            mandatory=True,
            usedefault=True,
        ),
        outlier_sensitivity=dict(
            argstr='--sat %.4f',
            mandatory=True,
            xor=['auto_detect_sensitivity'],
        ),
        scaled_intensity_outputs=dict(argstr='--iscaleout %s', ),
        subjects_dir=dict(),
        subsample_threshold=dict(argstr='--subsample %d', ),
        transform_outputs=dict(argstr='--lta %s', ),
    )
    inputs = RobustTemplate.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RobustTemplate_outputs():
    output_map = dict(
        out_file=dict(extensions=None, ),
        scaled_intensity_outputs=dict(),
        transform_outputs=dict(),
    )
    outputs = RobustTemplate.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import Surface2VolTransform


def test_Surface2VolTransform_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hemi=dict(
            argstr='--hemi %s',
            mandatory=True,
        ),
        mkmask=dict(
            argstr='--mkmask',
            xor=['source_file'],
        ),
        projfrac=dict(argstr='--projfrac %s', ),
        reg_file=dict(
            argstr='--volreg %s',
            extensions=None,
            mandatory=True,
            xor=['subject_id'],
        ),
        source_file=dict(
            argstr='--surfval %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            xor=['mkmask'],
        ),
        subject_id=dict(
            argstr='--identity %s',
            xor=['reg_file'],
        ),
        subjects_dir=dict(argstr='--sd %s', ),
        surf_name=dict(argstr='--surf %s', ),
        template_file=dict(
            argstr='--template %s',
            extensions=None,
        ),
        transformed_file=dict(
            argstr='--outvol %s',
            extensions=None,
            hash_files=False,
            name_source=['source_file'],
            name_template='%s_asVol.nii',
        ),
        vertexvol_file=dict(
            argstr='--vtxvol %s',
            extensions=None,
            hash_files=False,
            name_source=['source_file'],
            name_template='%s_asVol_vertex.nii',
        ),
    )
    inputs = Surface2VolTransform.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Surface2VolTransform_outputs():
    output_map = dict(
        transformed_file=dict(extensions=None, ),
        vertexvol_file=dict(extensions=None, ),
    )
    outputs = Surface2VolTransform.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .diffusion import *
from .featurecreator import GenerateCsfClippedFromClassifiedImage
from .segmentation import *
from .filtering import *
from .brains import *
from .testing import *
from .utilities import *
from .legacy import *
from .registration import *
from .converters import DWISimpleCompare, DWICompare



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..converters import DicomToNrrdConverter


def test_DicomToNrrdConverter_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputDicomDirectory=dict(argstr='--inputDicomDirectory %s', ),
        outputDirectory=dict(
            argstr='--outputDirectory %s',
            hash_files=False,
        ),
        outputVolume=dict(argstr='--outputVolume %s', ),
        smallGradientThreshold=dict(argstr='--smallGradientThreshold %f', ),
        useBMatrixGradientDirections=dict(
            argstr='--useBMatrixGradientDirections ', ),
        useIdentityMeaseurementFrame=dict(
            argstr='--useIdentityMeaseurementFrame ', ),
        writeProtocolGradientsFile=dict(
            argstr='--writeProtocolGradientsFile ', ),
    )
    inputs = DicomToNrrdConverter.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DicomToNrrdConverter_outputs():
    output_map = dict(outputDirectory=dict(), )
    outputs = DicomToNrrdConverter.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (CommandLine, CommandLineInputSpec, SEMLikeCommandLine,
                     TraitedSpec, File, Directory, traits, isdefined,
                     InputMultiPath, OutputMultiPath)


class maxcurvatureInputSpec(CommandLineInputSpec):
    image = File(desc="FA Image", exists=True, argstr="--image %s")
    output = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output File",
        argstr="--output %s")
    sigma = traits.Float(desc="Scale of Gradients", argstr="--sigma %f")
    verbose = traits.Bool(desc="produce verbose output", argstr="--verbose ")


class maxcurvatureOutputSpec(TraitedSpec):
    output = File(desc="Output File", exists=True)


class maxcurvature(SEMLikeCommandLine):
    """title: MaxCurvature-Hessian (DTIProcess)

category: Diffusion

description: This program computes the Hessian of the FA image (--image). We use this scalar image as a registration input when doing DTI atlas building. For most adult FA we use a sigma of 2 whereas for neonate or primate images and sigma of 1 or 1.5 is more appropriate. For really noisy images, 2.5 - 4 can be considered. The final image (--output) shows the main feature of the input image.

version: 1.1.0

documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/Nightly/Extensions/DTIProcess

license: Copyright (c)  Casey Goodlett. All rights reserved.
  See http://www.ia.unc.edu/dev/Copyright.htm for details.
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

contributor: Casey Goodlett

acknowledgements: Hans Johnson(1,3,4); Kent Williams(1); (1=University of Iowa Department of Psychiatry, 3=University of Iowa Department of Biomedical Engineering, 4=University of Iowa Department of Electrical and Computer Engineering) provided conversions to make DTIProcess compatible with Slicer execution, and simplified the stand-alone build requirements by removing the dependancies on boost and a fortran compiler.

"""

    input_spec = maxcurvatureInputSpec
    output_spec = maxcurvatureOutputSpec
    _cmd = " maxcurvature "
    _outputs_filenames = {'output': 'output.nii'}
    _redirect_x = False



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The stats module provides higher-level interfaces to some of the operations
that can be performed with the niftyseg stats (seg_stats) command-line program.
"""
from __future__ import print_function
import numpy as np

from ..base import TraitedSpec, File, traits, CommandLineInputSpec
from .base import NiftySegCommand
from ..niftyreg.base import get_custom_path


class StatsInput(CommandLineInputSpec):
    """Input Spec for seg_stats interfaces."""
    in_file = File(
        position=2,
        argstr='%s',
        exists=True,
        mandatory=True,
        desc='image to operate on')

    # Constrains
    mask_file = File(
        exists=True,
        position=-2,
        argstr='-m %s',
        desc='statistics within the masked area')

    desc = 'Only estimate statistics if voxel is larger than <float>'
    larger_voxel = traits.Float(argstr='-t %f', position=-3, desc=desc)


class StatsOutput(TraitedSpec):
    """Output Spec for seg_stats interfaces."""
    output = traits.Array(desc='Output array from seg_stats')


class StatsCommand(NiftySegCommand):
    """
    Base Command Interface for seg_stats interfaces.

    The executable seg_stats enables the estimation of image statistics on
    continuous voxel intensities (average, standard deviation, min/max, robust
    range, percentiles, sum, probabilistic volume, entropy, etc) either over
    the full image or on a per slice basis (slice axis can be specified),
    statistics over voxel coordinates (location of max, min and centre of
    mass, bounding box, etc) and statistics over categorical images (e.g. per
    region volume, count, average, Dice scores, etc). These statistics are
    robust to the presence of NaNs, and can be constrained by a mask and/or
    thresholded at a certain level.
    """
    _cmd = get_custom_path('seg_stats', env_dir='NIFTYSEGDIR')
    input_spec = StatsInput
    output_spec = StatsOutput

    def _parse_stdout(self, stdout):
        out = []
        for string_line in stdout.split("\n"):
            if string_line.startswith('#'):
                continue
            if len(string_line) <= 1:
                continue
            line = [float(s) for s in string_line.split()]
            out.append(line)
        return np.array(out).squeeze()

    def _run_interface(self, runtime):
        new_runtime = super(StatsCommand, self)._run_interface(runtime)
        self.output = self._parse_stdout(new_runtime.stdout)
        return new_runtime

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['output'] = self.output
        return outputs


class UnaryStatsInput(StatsInput):
    """Input Spec for seg_stats unary operations."""
    operation = traits.Enum(
        'r',
        'R',
        'a',
        's',
        'v',
        'vl',
        'vp',
        'n',
        'np',
        'e',
        'ne',
        'x',
        'X',
        'c',
        'B',
        'xvox',
        'xdim',
        argstr='-%s',
        position=4,
        mandatory=True,
        desc='operation to perform')


class UnaryStats(StatsCommand):
    """
    Interface for executable seg_stats from NiftySeg platform.

    Interface to use any unary statistical operations that can be performed

    with the seg_stats command-line program.

    See below for those operations::

    r - The range <min max> of all voxels.

    R - The robust range (assuming 2% outliers on both sides) of all voxels

    a - Average of all voxels

    s - Standard deviation of all voxels

    v - Volume of all voxels above 0 (<# voxels> * <volume per voxel>)

    vl - Volume of each integer label (<# voxels per label> * \
<volume per voxel>)

    vp - Volume of all probabilsitic voxels (sum(<in>) * <volume per voxel>)

    n - Count of all voxels above 0 (<# voxels>)

    np - Sum of all fuzzy voxels (sum(<in>))

    e - Entropy of all voxels

    ne - Normalized entropy of all voxels

    x - Location (i j k x y z) of the smallest value in the image

    X - Location (i j k x y z) of the largest value in the image

    c - Location (i j k x y z) of the centre of mass of the object

    B - Bounding box of all nonzero voxels [ xmin xsize ymin ysize zmin zsize ]

    xvox - Output the number of voxels in the x direction. Replace x with \
y/z for other directions.

    xdim - Output the voxel dimention in the x direction. Replace x with \
y/z for other directions.

    `Source code <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg>`_ |
    `Documentation <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg_documentation>`_

    Examples
    --------
    >>> import copy
    >>> from nipype.interfaces import niftyseg
    >>> unary = niftyseg.UnaryStats()
    >>> unary.inputs.in_file = 'im1.nii'
    >>> # Test v operation
    >>> unary_v = copy.deepcopy(unary)
    >>> unary_v.inputs.operation = 'v'
    >>> unary_v.cmdline
    'seg_stats im1.nii -v'
    >>> unary_v.run()  # doctest: +SKIP
    >>> # Test vl operation
    >>> unary_vl = copy.deepcopy(unary)
    >>> unary_vl.inputs.operation = 'vl'
    >>> unary_vl.cmdline
    'seg_stats im1.nii -vl'
    >>> unary_vl.run()  # doctest: +SKIP
    >>> # Test x operation
    >>> unary_x = copy.deepcopy(unary)
    >>> unary_x.inputs.operation = 'x'
    >>> unary_x.cmdline
    'seg_stats im1.nii -x'
    >>> unary_x.run()  # doctest: +SKIP

    """
    input_spec = UnaryStatsInput


class BinaryStatsInput(StatsInput):
    """Input Spec for seg_stats Binary operations."""
    operation = traits.Enum(
        'p',
        'sa',
        'ss',
        'svp',
        'al',
        'd',
        'ncc',
        'nmi',
        'Vl',
        'Nl',
        mandatory=True,
        argstr='-%s',
        position=4,
        desc='operation to perform')

    operand_file = File(
        exists=True,
        argstr="%s",
        mandatory=True,
        position=5,
        xor=["operand_value"],
        desc="second image to perform operation with")

    operand_value = traits.Float(
        argstr='%.8f',
        mandatory=True,
        position=5,
        xor=["operand_file"],
        desc='value to perform operation with')


class BinaryStats(StatsCommand):
    """
    Interface for executable seg_stats from NiftySeg platform.

    Interface to use any binary statistical operations that can be performed

    with the seg_stats command-line program.

    See below for those operations::

    p - <float> - The <float>th percentile of all voxels intensity \
(float=[0,100])

    sa - <ax> - Average of all voxels

    ss - <ax> - Standard deviation of all voxels

    svp - <ax> - Volume of all probabilsitic voxels (sum(<in>) * \
<volume per voxel>)

    al - <in2> - Average value in <in> for each label in <in2>

    d - <in2> - Calculate the Dice score between all classes in <in>\
and <in2>

    ncc - <in2> - Normalized cross correlation between <in> and <in2>

    nmi - <in2> - Normalized Mutual Information between <in> and <in2>

    Vl - <csv> - Volume of each integer label <in>. Save to <csv>file.

    Nl - <csv> - Count of each label <in>. Save to <csv> file.

    `Source code <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg>`_ |
    `Documentation <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg_documentation>`_

    Examples
    --------
    >>> import copy
    >>> from nipype.interfaces import niftyseg
    >>> binary = niftyseg.BinaryStats()
    >>> binary.inputs.in_file = 'im1.nii'
    >>> # Test sa operation
    >>> binary_sa = copy.deepcopy(binary)
    >>> binary_sa.inputs.operation = 'sa'
    >>> binary_sa.inputs.operand_value = 2.0
    >>> binary_sa.cmdline
    'seg_stats im1.nii -sa 2.00000000'
    >>> binary_sa.run()  # doctest: +SKIP
    >>> # Test ncc operation
    >>> binary_ncc = copy.deepcopy(binary)
    >>> binary_ncc.inputs.operation = 'ncc'
    >>> binary_ncc.inputs.operand_file = 'im2.nii'
    >>> binary_ncc.cmdline
    'seg_stats im1.nii -ncc im2.nii'
    >>> binary_ncc.run()  # doctest: +SKIP
    >>> # Test Nl operation
    >>> binary_nl = copy.deepcopy(binary)
    >>> binary_nl.inputs.operation = 'Nl'
    >>> binary_nl.inputs.operand_file = 'output.csv'
    >>> binary_nl.cmdline
    'seg_stats im1.nii -Nl output.csv'
    >>> binary_nl.run()  # doctest: +SKIP

    """
    input_spec = BinaryStatsInput



# -*- coding: utf-8 -*-
"""Parallel workflow execution via PBS/Torque
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
import sys

from .base import (GraphPluginBase, logger)

soma_not_loaded = False
try:
    from soma.workflow.client import (Job, Workflow, WorkflowController,
                                      Helper)
except:
    soma_not_loaded = True


class SomaFlowPlugin(GraphPluginBase):
    """Execute using Soma workflow
    """

    def __init__(self, plugin_args=None):
        if soma_not_loaded:
            raise ImportError('SomaFlow could not be imported')
        super(SomaFlowPlugin, self).__init__(plugin_args=plugin_args)

    def _submit_graph(self, pyfiles, dependencies, nodes):
        jobs = []
        soma_deps = []
        for idx, fname in enumerate(pyfiles):
            name = os.path.splitext(os.path.split(fname)[1])[0]
            jobs.append(Job(command=[sys.executable, fname], name=name))
        for key, values in list(dependencies.items()):
            for val in values:
                soma_deps.append((jobs[val], jobs[key]))

        wf = Workflow(jobs, soma_deps)
        logger.info('serializing workflow')
        Helper.serialize('workflow', wf)
        controller = WorkflowController()
        logger.info('submitting workflow')
        wf_id = controller.submit_workflow(wf)
        Helper.wait_workflow(wf_id, controller)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import ReHo


def test_ReHo_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        chi_sq=dict(argstr='-chi_sq', ),
        ellipsoid=dict(
            argstr='-neigh_X %s -neigh_Y %s -neigh_Z %s',
            xor=['sphere', 'neighborhood'],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-inset %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        label_set=dict(
            argstr='-in_rois %s',
            extensions=None,
        ),
        mask_file=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        neighborhood=dict(
            argstr='-nneigh %s',
            xor=['sphere', 'ellipsoid'],
        ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            keep_extension=True,
            name_source='in_file',
            name_template='%s_reho',
            position=0,
        ),
        overwrite=dict(argstr='-overwrite', ),
        sphere=dict(
            argstr='-neigh_RAD %s',
            xor=['neighborhood', 'ellipsoid'],
        ),
    )
    inputs = ReHo.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ReHo_outputs():
    output_map = dict(
        out_file=dict(extensions=None, ),
        out_vals=dict(extensions=None, ),
    )
    outputs = ReHo.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dti import DistanceMap


def test_DistanceMap_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        distance_map=dict(
            argstr='--out=%s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='--in=%s',
            extensions=None,
            mandatory=True,
        ),
        invert_input=dict(argstr='--invert', ),
        local_max_file=dict(
            argstr='--localmax=%s',
            hash_files=False,
        ),
        mask_file=dict(
            argstr='--mask=%s',
            extensions=None,
        ),
        output_type=dict(),
    )
    inputs = DistanceMap.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DistanceMap_outputs():
    output_map = dict(
        distance_map=dict(extensions=None, ),
        local_max_file=dict(extensions=None, ),
    )
    outputs = DistanceMap.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsuite import Pvc


def test_Pvc_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputMRIFile=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        inputMaskFile=dict(
            argstr='-m %s',
            extensions=None,
        ),
        outputLabelFile=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
        ),
        outputTissueFractionFile=dict(
            argstr='-f %s',
            extensions=None,
            genfile=True,
        ),
        spatialPrior=dict(argstr='-l %f', ),
        threeClassFlag=dict(argstr='-3', ),
        timer=dict(argstr='--timer', ),
        verbosity=dict(argstr='-v %d', ),
    )
    inputs = Pvc.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Pvc_outputs():
    output_map = dict(
        outputLabelFile=dict(extensions=None, ),
        outputTissueFractionFile=dict(extensions=None, ),
    )
    outputs = Pvc.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



from __future__ import unicode_literals
from ..nbs import NetworkBasedStatistic
from ....utils.misc import package_check
import numpy as np
import networkx as nx
import pytest

have_cv = True
try:
    package_check('cviewer')
except Exception as e:
    have_cv = False


@pytest.fixture()
def creating_graphs(tmpdir):
    graphlist = []
    graphnames = ["name" + str(i) for i in range(6)]
    for idx, name in enumerate(graphnames):
        graph = np.random.rand(10, 10)
        G = nx.from_numpy_matrix(graph)
        out_file = tmpdir.strpath + graphnames[idx] + '.pck'
        # Save as pck file
        nx.write_gpickle(G, out_file)
        graphlist.append(out_file)
    return graphlist


@pytest.mark.skipif(
    have_cv, reason="tests for import error, cviewer available")
def test_importerror(creating_graphs, tmpdir):
    tmpdir.chdir()
    graphlist = creating_graphs
    group1 = graphlist[:3]
    group2 = graphlist[3:]

    nbs = NetworkBasedStatistic()
    nbs.inputs.in_group1 = group1
    nbs.inputs.in_group2 = group2
    nbs.inputs.edge_key = "weight"

    with pytest.raises(ImportError) as e:
        nbs.run()


@pytest.mark.skipif(not have_cv, reason="cviewer has to be available")
def test_keyerror(creating_graphs):
    graphlist = creating_graphs

    group1 = graphlist[:3]
    group2 = graphlist[3:]

    nbs = NetworkBasedStatistic()
    nbs.inputs.in_group1 = group1
    nbs.inputs.in_group2 = group2
    nbs.inputs.edge_key = "Your_edge"

    with pytest.raises(KeyError) as e:
        nbs.run()
    assert "the graph edges do not have Your_edge attribute" in str(e.value)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..reconstruction import RESTORE


def test_RESTORE_inputs():
    input_map = dict(
        b0_thres=dict(usedefault=True, ),
        in_bval=dict(
            extensions=None,
            mandatory=True,
        ),
        in_bvec=dict(
            extensions=None,
            mandatory=True,
        ),
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        in_mask=dict(extensions=None, ),
        noise_mask=dict(extensions=None, ),
        out_prefix=dict(),
    )
    inputs = RESTORE.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RESTORE_outputs():
    output_map = dict(
        evals=dict(extensions=None, ),
        evecs=dict(extensions=None, ),
        fa=dict(extensions=None, ),
        md=dict(extensions=None, ),
        mode=dict(extensions=None, ),
        rd=dict(extensions=None, ),
        trace=dict(extensions=None, ),
    )
    outputs = RESTORE.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""Provides interfaces to various commands provided by dcmstack
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
from os import path as op
import string
import errno
from glob import glob

import nibabel as nb
import imghdr

from .base import (TraitedSpec, DynamicTraitedSpec, InputMultiPath, File,
                   Directory, traits, BaseInterface, isdefined, Undefined)
from ..utils import NUMPY_MMAP

have_dcmstack = True
try:
    import dicom
    import dcmstack
    from dcmstack.dcmmeta import NiftiWrapper
except ImportError:
    have_dcmstack = False


def sanitize_path_comp(path_comp):
    result = []
    for char in path_comp:
        if char not in string.letters + string.digits + '-_.':
            result.append('_')
        else:
            result.append(char)
    return ''.join(result)


class NiftiGeneratorBaseInputSpec(TraitedSpec):
    out_format = traits.Str(desc="String which can be formatted with "
                            "meta data to create the output filename(s)")
    out_ext = traits.Str(
        '.nii.gz', usedefault=True, desc="Determines output file type")
    out_path = Directory(
        desc='output path, current working directory if not set')


class NiftiGeneratorBase(BaseInterface):
    '''Base class for interfaces that produce Nifti files, potentially with
    embedded meta data.'''

    def _get_out_path(self, meta, idx=None):
        '''Return the output path for the gernerated Nifti.'''
        if self.inputs.out_format:
            out_fmt = self.inputs.out_format
        else:
            # If no out_format is specified, use a sane default that will work
            # with the provided meta data.
            out_fmt = []
            if idx is not None:
                out_fmt.append('%03d' % idx)
            if 'SeriesNumber' in meta:
                out_fmt.append('%(SeriesNumber)03d')
            if 'ProtocolName' in meta:
                out_fmt.append('%(ProtocolName)s')
            elif 'SeriesDescription' in meta:
                out_fmt.append('%(SeriesDescription)s')
            else:
                out_fmt.append('sequence')
            out_fmt = '-'.join(out_fmt)
        out_fn = (out_fmt % meta) + self.inputs.out_ext
        out_fn = sanitize_path_comp(out_fn)

        out_path = os.getcwd()
        if isdefined(self.inputs.out_path):
            out_path = op.abspath(self.inputs.out_path)

            # now, mkdir -p $out_path
            try:
                os.makedirs(out_path)
            except OSError as exc:  # Python >2.5
                if exc.errno == errno.EEXIST and op.isdir(out_path):
                    pass
                else:
                    raise

        return op.join(out_path, out_fn)


class DcmStackInputSpec(NiftiGeneratorBaseInputSpec):
    dicom_files = traits.Either(
        InputMultiPath(File(exists=True)),
        Directory(exists=True),
        traits.Str(),
        mandatory=True)
    embed_meta = traits.Bool(desc="Embed DICOM meta data into result")
    exclude_regexes = traits.List(desc="Meta data to exclude, suplementing "
                                  "any default exclude filters")
    include_regexes = traits.List(desc="Meta data to include, overriding any "
                                  "exclude filters")
    force_read = traits.Bool(
        True,
        usedefault=True,
        desc=('Force reading files without DICM marker'))


class DcmStackOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class DcmStack(NiftiGeneratorBase):
    '''Create one Nifti file from a set of DICOM files. Can optionally embed
    meta data.

    Example
    -------

    >>> from nipype.interfaces.dcmstack import DcmStack
    >>> stacker = DcmStack()
    >>> stacker.inputs.dicom_files = 'path/to/series/'
    >>> stacker.run() # doctest: +SKIP
    >>> result.outputs.out_file # doctest: +SKIP
    '/path/to/cwd/sequence.nii.gz'
    '''
    input_spec = DcmStackInputSpec
    output_spec = DcmStackOutputSpec

    def _get_filelist(self, trait_input):
        if isinstance(trait_input, (str, bytes)):
            if op.isdir(trait_input):
                return glob(op.join(trait_input, '*.dcm'))
            else:
                return glob(trait_input)

        return trait_input

    def _run_interface(self, runtime):
        src_paths = self._get_filelist(self.inputs.dicom_files)
        include_regexes = dcmstack.default_key_incl_res
        if isdefined(self.inputs.include_regexes):
            include_regexes += self.inputs.include_regexes
        exclude_regexes = dcmstack.default_key_excl_res
        if isdefined(self.inputs.exclude_regexes):
            exclude_regexes += self.inputs.exclude_regexes
        meta_filter = dcmstack.make_key_regex_filter(exclude_regexes,
                                                     include_regexes)
        stack = dcmstack.DicomStack(meta_filter=meta_filter)
        for src_path in src_paths:
            if not imghdr.what(src_path) == "gif":
                src_dcm = dicom.read_file(
                    src_path, force=self.inputs.force_read)
                stack.add_dcm(src_dcm)
        nii = stack.to_nifti(embed_meta=True)
        nw = NiftiWrapper(nii)
        self.out_path = \
            self._get_out_path(nw.meta_ext.get_class_dict(('global', 'const')))
        if not self.inputs.embed_meta:
            nw.remove_extension()
        nb.save(nii, self.out_path)
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["out_file"] = self.out_path
        return outputs


class GroupAndStackOutputSpec(TraitedSpec):
    out_list = traits.List(desc="List of output nifti files")


class GroupAndStack(DcmStack):
    '''Create (potentially) multiple Nifti files for a set of DICOM files.
    '''
    input_spec = DcmStackInputSpec
    output_spec = GroupAndStackOutputSpec

    def _run_interface(self, runtime):
        src_paths = self._get_filelist(self.inputs.dicom_files)
        stacks = dcmstack.parse_and_stack(src_paths)

        self.out_list = []
        for key, stack in list(stacks.items()):
            nw = NiftiWrapper(stack.to_nifti(embed_meta=True))
            const_meta = nw.meta_ext.get_class_dict(('global', 'const'))
            out_path = self._get_out_path(const_meta)
            if not self.inputs.embed_meta:
                nw.remove_extension()
            nb.save(nw.nii_img, out_path)
            self.out_list.append(out_path)

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["out_list"] = self.out_list
        return outputs


class LookupMetaInputSpec(TraitedSpec):
    in_file = File(mandatory=True, exists=True, desc='The input Nifti file')
    meta_keys = traits.Either(
        traits.List(),
        traits.Dict(),
        mandatory=True,
        desc=("List of meta data keys to lookup, or a "
              "dict where keys specify the meta data "
              "keys to lookup and the values specify "
              "the output names"))


class LookupMeta(BaseInterface):
    '''Lookup meta data values from a Nifti with embedded meta data.

    Example
    -------

    >>> from nipype.interfaces import dcmstack
    >>> lookup = dcmstack.LookupMeta()
    >>> lookup.inputs.in_file = 'functional.nii'
    >>> lookup.inputs.meta_keys = {'RepetitionTime' : 'TR', \
                                   'EchoTime' : 'TE'}
    >>> result = lookup.run() # doctest: +SKIP
    >>> result.outputs.TR # doctest: +SKIP
    9500.0
    >>> result.outputs.TE # doctest: +SKIP
    95.0
    '''
    input_spec = LookupMetaInputSpec
    output_spec = DynamicTraitedSpec

    def _make_name_map(self):
        if isinstance(self.inputs.meta_keys, list):
            self._meta_keys = {}
            for key in self.inputs.meta_keys:
                self._meta_keys[key] = key
        else:
            self._meta_keys = self.inputs.meta_keys

    def _outputs(self):
        self._make_name_map()
        outputs = super(LookupMeta, self)._outputs()
        undefined_traits = {}
        for out_name in list(self._meta_keys.values()):
            outputs.add_trait(out_name, traits.Any)
            undefined_traits[out_name] = Undefined
        outputs.trait_set(trait_change_notify=False, **undefined_traits)
        # Not sure why this is needed
        for out_name in list(self._meta_keys.values()):
            _ = getattr(outputs, out_name)
        return outputs

    def _run_interface(self, runtime):
        # If the 'meta_keys' input is a list, covert it to a dict
        self._make_name_map()
        nw = NiftiWrapper.from_filename(self.inputs.in_file)
        self.result = {}
        for meta_key, out_name in list(self._meta_keys.items()):
            self.result[out_name] = nw.meta_ext.get_values(meta_key)

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs.update(self.result)
        return outputs


class CopyMetaInputSpec(TraitedSpec):
    src_file = File(mandatory=True, exists=True)
    dest_file = File(mandatory=True, exists=True)
    include_classes = traits.List(desc="List of specific meta data "
                                  "classifications to include. If not "
                                  "specified include everything.")
    exclude_classes = traits.List(desc="List of meta data "
                                  "classifications to exclude")


class CopyMetaOutputSpec(TraitedSpec):
    dest_file = File(exists=True)


class CopyMeta(BaseInterface):
    '''Copy meta data from one Nifti file to another. Useful for preserving
    meta data after some processing steps.'''
    input_spec = CopyMetaInputSpec
    output_spec = CopyMetaOutputSpec

    def _run_interface(self, runtime):
        src_nii = nb.load(self.inputs.src_file)
        src = NiftiWrapper(src_nii, make_empty=True)
        dest_nii = nb.load(self.inputs.dest_file)
        dest = NiftiWrapper(dest_nii, make_empty=True)
        classes = src.meta_ext.get_valid_classes()
        if self.inputs.include_classes:
            classes = [
                cls for cls in classes if cls in self.inputs.include_classes
            ]
        if self.inputs.exclude_classes:
            classes = [
                cls for cls in classes
                if cls not in self.inputs.exclude_classes
            ]

        for cls in classes:
            src_dict = src.meta_ext.get_class_dict(cls)
            dest_dict = dest.meta_ext.get_class_dict(cls)
            dest_dict.update(src_dict)
        # Update the shape and slice dimension to reflect the meta extension
        # update.
        dest.meta_ext.slice_dim = src.meta_ext.slice_dim
        dest.meta_ext.shape = src.meta_ext.shape

        self.out_path = op.join(os.getcwd(), op.basename(
            self.inputs.dest_file))
        dest.to_filename(self.out_path)

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['dest_file'] = self.out_path
        return outputs


class MergeNiftiInputSpec(NiftiGeneratorBaseInputSpec):
    in_files = traits.List(mandatory=True, desc="List of Nifti files to merge")
    sort_order = traits.Either(
        traits.Str(),
        traits.List(),
        desc="One or more meta data keys to "
        "sort files by.")
    merge_dim = traits.Int(desc="Dimension to merge along. If not "
                           "specified, the last singular or "
                           "non-existant dimension is used.")


class MergeNiftiOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc="Merged Nifti file")


def make_key_func(meta_keys, index=None):
    def key_func(src_nii):
        result = [src_nii.get_meta(key, index) for key in meta_keys]
        return result

    return key_func


class MergeNifti(NiftiGeneratorBase):
    '''Merge multiple Nifti files into one. Merges together meta data
    extensions as well.'''
    input_spec = MergeNiftiInputSpec
    output_spec = MergeNiftiOutputSpec

    def _run_interface(self, runtime):
        niis = [nb.load(fn, mmap=NUMPY_MMAP) for fn in self.inputs.in_files]
        nws = [NiftiWrapper(nii, make_empty=True) for nii in niis]
        if self.inputs.sort_order:
            sort_order = self.inputs.sort_order
            if isinstance(sort_order, (str, bytes)):
                sort_order = [sort_order]
            nws.sort(key=make_key_func(sort_order))
        if self.inputs.merge_dim == traits.Undefined:
            merge_dim = None
        else:
            merge_dim = self.inputs.merge_dim
        merged = NiftiWrapper.from_sequence(nws, merge_dim)
        const_meta = merged.meta_ext.get_class_dict(('global', 'const'))
        self.out_path = self._get_out_path(const_meta)
        nb.save(merged.nii_img, self.out_path)
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['out_file'] = self.out_path
        return outputs


class SplitNiftiInputSpec(NiftiGeneratorBaseInputSpec):
    in_file = File(exists=True, mandatory=True, desc="Nifti file to split")
    split_dim = traits.Int(desc="Dimension to split along. If not "
                           "specified, the last dimension is used.")


class SplitNiftiOutputSpec(TraitedSpec):
    out_list = traits.List(File(exists=True), desc="Split Nifti files")


class SplitNifti(NiftiGeneratorBase):
    '''
    Split one Nifti file into many along the specified dimension. Each
    result has an updated meta data extension as well.
    '''
    input_spec = SplitNiftiInputSpec
    output_spec = SplitNiftiOutputSpec

    def _run_interface(self, runtime):
        self.out_list = []
        nii = nb.load(self.inputs.in_file)
        nw = NiftiWrapper(nii, make_empty=True)
        split_dim = None
        if self.inputs.split_dim == traits.Undefined:
            split_dim = None
        else:
            split_dim = self.inputs.split_dim
        for split_idx, split_nw in enumerate(nw.split(split_dim)):
            const_meta = split_nw.meta_ext.get_class_dict(('global', 'const'))
            out_path = self._get_out_path(const_meta, idx=split_idx)
            nb.save(split_nw.nii_img, out_path)
            self.out_list.append(out_path)

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['out_list'] = self.out_list
        return outputs



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Top-level namespace for ants."""

# Registraiton programs
from .registration import (ANTS, Registration, RegistrationSynQuick,
                           CompositeTransformUtil, MeasureImageSimilarity)

# Resampling Programs
from .resampling import (ApplyTransforms, ApplyTransformsToPoints,
                         WarpImageMultiTransform,
                         WarpTimeSeriesImageMultiTransform)

# Segmentation Programs
from .segmentation import (Atropos, LaplacianThickness, N4BiasFieldCorrection,
                           JointFusion, CorticalThickness, BrainExtraction,
                           DenoiseImage, AntsJointFusion)

# Visualization Programs
from .visualization import ConvertScalarImageToRGB, CreateTiledMosaic

# Utility Programs
from .utils import (AverageAffineTransform, AverageImages, MultiplyImages,
                    CreateJacobianDeterminantImage, AffineInitializer,
                    ComposeMultiTransform, LabelGeometry)



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Provides interfaces to various longitudinal commands provided by freesurfer
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os

from ... import logging
from ..base import (TraitedSpec, File, traits, InputMultiPath, OutputMultiPath,
                    isdefined)
from .base import (FSCommand, FSTraitedSpec, FSCommandOpenMP,
                   FSTraitedSpecOpenMP)

__docformat__ = 'restructuredtext'
iflogger = logging.getLogger('nipype.interface')


class RobustTemplateInputSpec(FSTraitedSpecOpenMP):
    # required
    in_files = InputMultiPath(
        File(exists=True),
        mandatory=True,
        argstr='--mov %s',
        desc='input movable volumes to be aligned to common mean/median '
        'template')
    out_file = File(
        'mri_robust_template_out.mgz',
        mandatory=True,
        usedefault=True,
        argstr='--template %s',
        desc='output template volume (final mean/median image)')
    auto_detect_sensitivity = traits.Bool(
        argstr='--satit',
        xor=['outlier_sensitivity'],
        mandatory=True,
        desc='auto-detect good sensitivity (recommended for head or full '
        'brain scans)')
    outlier_sensitivity = traits.Float(
        argstr='--sat %.4f',
        xor=['auto_detect_sensitivity'],
        mandatory=True,
        desc='set outlier sensitivity manually (e.g. "--sat 4.685" ). Higher '
        'values mean less sensitivity.')
    # optional
    transform_outputs = traits.Either(
        InputMultiPath(File(exists=False)),
        traits.Bool,
        argstr='--lta %s',
        desc='output xforms to template (for each input)')
    intensity_scaling = traits.Bool(
        default_value=False,
        argstr='--iscale',
        desc='allow also intensity scaling (default off)')
    scaled_intensity_outputs = traits.Either(
        InputMultiPath(File(exists=False)),
        traits.Bool,
        argstr='--iscaleout %s',
        desc='final intensity scales (will activate --iscale)')
    subsample_threshold = traits.Int(
        argstr='--subsample %d',
        desc='subsample if dim > # on all axes (default no subs.)')
    average_metric = traits.Enum(
        'median',
        'mean',
        argstr='--average %d',
        desc='construct template from: 0 Mean, 1 Median (default)')
    initial_timepoint = traits.Int(
        argstr='--inittp %d',
        desc='use TP# for spacial init (default random), 0: no init')
    fixed_timepoint = traits.Bool(
        default_value=False,
        argstr='--fixtp',
        desc='map everthing to init TP# (init TP is not resampled)')
    no_iteration = traits.Bool(
        default_value=False,
        argstr='--noit',
        desc='do not iterate, just create first template')
    initial_transforms = InputMultiPath(
        File(exists=True),
        argstr='--ixforms %s',
        desc='use initial transforms (lta) on source')
    in_intensity_scales = InputMultiPath(
        File(exists=True),
        argstr='--iscalein %s',
        desc='use initial intensity scales')


class RobustTemplateOutputSpec(TraitedSpec):
    out_file = File(
        exists=True, desc='output template volume (final mean/median image)')
    transform_outputs = OutputMultiPath(
        File(exists=True), desc="output xform files from moving to template")
    scaled_intensity_outputs = OutputMultiPath(
        File(exists=True), desc="output final intensity scales")


class RobustTemplate(FSCommandOpenMP):
    """ construct an unbiased robust template for longitudinal volumes

    Examples
    --------
    >>> from nipype.interfaces.freesurfer import RobustTemplate
    >>> template = RobustTemplate()
    >>> template.inputs.in_files = ['structural.nii', 'functional.nii']
    >>> template.inputs.auto_detect_sensitivity = True
    >>> template.inputs.average_metric = 'mean'
    >>> template.inputs.initial_timepoint = 1
    >>> template.inputs.fixed_timepoint = True
    >>> template.inputs.no_iteration = True
    >>> template.inputs.subsample_threshold = 200
    >>> template.cmdline  #doctest:
    'mri_robust_template --satit --average 0 --fixtp --mov structural.nii functional.nii --inittp 1 --noit --template mri_robust_template_out.mgz --subsample 200'
    >>> template.inputs.out_file = 'T1.nii'
    >>> template.cmdline  #doctest:
    'mri_robust_template --satit --average 0 --fixtp --mov structural.nii functional.nii --inittp 1 --noit --template T1.nii --subsample 200'

    >>> template.inputs.transform_outputs = ['structural.lta',
    ...                                      'functional.lta']
    >>> template.inputs.scaled_intensity_outputs = ['structural-iscale.txt',
    ...                                             'functional-iscale.txt']
    >>> template.cmdline    #doctest: +ELLIPSIS
    'mri_robust_template --satit --average 0 --fixtp --mov structural.nii functional.nii --inittp 1 --noit --template T1.nii --iscaleout .../structural-iscale.txt .../functional-iscale.txt --subsample 200 --lta .../structural.lta .../functional.lta'

    >>> template.inputs.transform_outputs = True
    >>> template.inputs.scaled_intensity_outputs = True
    >>> template.cmdline    #doctest: +ELLIPSIS
    'mri_robust_template --satit --average 0 --fixtp --mov structural.nii functional.nii --inittp 1 --noit --template T1.nii --iscaleout .../is1.txt .../is2.txt --subsample 200 --lta .../tp1.lta .../tp2.lta'

    >>> template.run()  #doctest: +SKIP

    References
    ----------
    [https://surfer.nmr.mgh.harvard.edu/fswiki/mri_robust_template]

    """

    _cmd = 'mri_robust_template'
    input_spec = RobustTemplateInputSpec
    output_spec = RobustTemplateOutputSpec

    def _format_arg(self, name, spec, value):
        if name == 'average_metric':
            # return enumeration value
            return spec.argstr % {"mean": 0, "median": 1}[value]
        if name in ('transform_outputs', 'scaled_intensity_outputs'):
            value = self._list_outputs()[name]
        return super(RobustTemplate, self)._format_arg(name, spec, value)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = os.path.abspath(self.inputs.out_file)
        n_files = len(self.inputs.in_files)
        fmt = '{}{:02d}.{}' if n_files > 9 else '{}{:d}.{}'
        if isdefined(self.inputs.transform_outputs):
            fnames = self.inputs.transform_outputs
            if fnames is True:
                fnames = [
                    fmt.format('tp', i + 1, 'lta') for i in range(n_files)
                ]
            outputs['transform_outputs'] = [os.path.abspath(x) for x in fnames]
        if isdefined(self.inputs.scaled_intensity_outputs):
            fnames = self.inputs.scaled_intensity_outputs
            if fnames is True:
                fnames = [
                    fmt.format('is', i + 1, 'txt') for i in range(n_files)
                ]
            outputs['scaled_intensity_outputs'] = [
                os.path.abspath(x) for x in fnames
            ]
        return outputs


class FuseSegmentationsInputSpec(FSTraitedSpec):
    # required
    subject_id = traits.String(
        argstr='%s', position=-3, desc="subject_id being processed")
    timepoints = InputMultiPath(
        traits.String(),
        mandatory=True,
        argstr='%s',
        position=-2,
        desc='subject_ids or timepoints to be processed')
    out_file = File(
        exists=False,
        mandatory=True,
        position=-1,
        desc="output fused segmentation file")
    in_segmentations = InputMultiPath(
        File(exists=True),
        argstr="-a %s",
        mandatory=True,
        desc="name of aseg file to use (default: aseg.mgz) \
        must include the aseg files for all the given timepoints")
    in_segmentations_noCC = InputMultiPath(
        File(exists=True),
        argstr="-c %s",
        mandatory=True,
        desc="name of aseg file w/o CC labels (default: aseg.auto_noCCseg.mgz) \
        must include the corresponding file for all the given timepoints")
    in_norms = InputMultiPath(
        File(exists=True),
        argstr="-n %s",
        mandatory=True,
        desc="-n <filename>  - name of norm file to use (default: norm.mgs) \
        must include the corresponding norm file for all given timepoints \
        as well as for the current subject")


class FuseSegmentationsOutputSpec(TraitedSpec):
    out_file = File(exists=False, desc="output fused segmentation file")


class FuseSegmentations(FSCommand):
    """ fuse segmentations together from multiple timepoints

    Examples
    --------
    >>> from nipype.interfaces.freesurfer import FuseSegmentations
    >>> fuse = FuseSegmentations()
    >>> fuse.inputs.subject_id = 'tp.long.A.template'
    >>> fuse.inputs.timepoints = ['tp1', 'tp2']
    >>> fuse.inputs.out_file = 'aseg.fused.mgz'
    >>> fuse.inputs.in_segmentations = ['aseg.mgz', 'aseg.mgz']
    >>> fuse.inputs.in_segmentations_noCC = ['aseg.mgz', 'aseg.mgz']
    >>> fuse.inputs.in_norms = ['norm.mgz', 'norm.mgz', 'norm.mgz']
    >>> fuse.cmdline
    'mri_fuse_segmentations -n norm.mgz -a aseg.mgz -c aseg.mgz tp.long.A.template tp1 tp2'
    """

    _cmd = 'mri_fuse_segmentations'
    input_spec = FuseSegmentationsInputSpec
    output_spec = FuseSegmentationsOutputSpec

    def _format_arg(self, name, spec, value):
        if name in ('in_segmentations', 'in_segmentations_noCC', 'in_norms'):
            # return enumeration value
            return spec.argstr % os.path.basename(value[0])
        return super(FuseSegmentations, self)._format_arg(name, spec, value)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = os.path.abspath(self.inputs.out_file)
        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import Complex


def test_Complex_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        complex_cartesian=dict(
            argstr='-complex',
            position=1,
            xor=[
                'real_polar', 'real_cartesian', 'complex_cartesian',
                'complex_polar', 'complex_split', 'complex_merge'
            ],
        ),
        complex_in_file=dict(
            argstr='%s',
            extensions=None,
            position=2,
        ),
        complex_in_file2=dict(
            argstr='%s',
            extensions=None,
            position=3,
        ),
        complex_merge=dict(
            argstr='-complexmerge',
            position=1,
            xor=[
                'real_polar', 'real_cartesian', 'complex_cartesian',
                'complex_polar', 'complex_split', 'complex_merge', 'start_vol',
                'end_vol'
            ],
        ),
        complex_out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=-3,
            xor=[
                'complex_out_file', 'magnitude_out_file', 'phase_out_file',
                'real_out_file', 'imaginary_out_file', 'real_polar',
                'real_cartesian'
            ],
        ),
        complex_polar=dict(
            argstr='-complexpolar',
            position=1,
            xor=[
                'real_polar', 'real_cartesian', 'complex_cartesian',
                'complex_polar', 'complex_split', 'complex_merge'
            ],
        ),
        complex_split=dict(
            argstr='-complexsplit',
            position=1,
            xor=[
                'real_polar', 'real_cartesian', 'complex_cartesian',
                'complex_polar', 'complex_split', 'complex_merge'
            ],
        ),
        end_vol=dict(
            argstr='%d',
            position=-1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        imaginary_in_file=dict(
            argstr='%s',
            extensions=None,
            position=3,
        ),
        imaginary_out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=-3,
            xor=[
                'complex_out_file', 'magnitude_out_file', 'phase_out_file',
                'real_polar', 'complex_cartesian', 'complex_polar',
                'complex_split', 'complex_merge'
            ],
        ),
        magnitude_in_file=dict(
            argstr='%s',
            extensions=None,
            position=2,
        ),
        magnitude_out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=-4,
            xor=[
                'complex_out_file', 'real_out_file', 'imaginary_out_file',
                'real_cartesian', 'complex_cartesian', 'complex_polar',
                'complex_split', 'complex_merge'
            ],
        ),
        output_type=dict(),
        phase_in_file=dict(
            argstr='%s',
            extensions=None,
            position=3,
        ),
        phase_out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=-3,
            xor=[
                'complex_out_file', 'real_out_file', 'imaginary_out_file',
                'real_cartesian', 'complex_cartesian', 'complex_polar',
                'complex_split', 'complex_merge'
            ],
        ),
        real_cartesian=dict(
            argstr='-realcartesian',
            position=1,
            xor=[
                'real_polar', 'real_cartesian', 'complex_cartesian',
                'complex_polar', 'complex_split', 'complex_merge'
            ],
        ),
        real_in_file=dict(
            argstr='%s',
            extensions=None,
            position=2,
        ),
        real_out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=-4,
            xor=[
                'complex_out_file', 'magnitude_out_file', 'phase_out_file',
                'real_polar', 'complex_cartesian', 'complex_polar',
                'complex_split', 'complex_merge'
            ],
        ),
        real_polar=dict(
            argstr='-realpolar',
            position=1,
            xor=[
                'real_polar', 'real_cartesian', 'complex_cartesian',
                'complex_polar', 'complex_split', 'complex_merge'
            ],
        ),
        start_vol=dict(
            argstr='%d',
            position=-2,
        ),
    )
    inputs = Complex.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Complex_outputs():
    output_map = dict(
        complex_out_file=dict(extensions=None, ),
        imaginary_out_file=dict(extensions=None, ),
        magnitude_out_file=dict(extensions=None, ),
        phase_out_file=dict(extensions=None, ),
        real_out_file=dict(extensions=None, ),
    )
    outputs = Complex.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import ApplyXFM


def test_ApplyXFM_inputs():
    input_map = dict(
        angle_rep=dict(argstr='-anglerep %s', ),
        apply_isoxfm=dict(
            argstr='-applyisoxfm %f',
            xor=['apply_xfm'],
        ),
        apply_xfm=dict(
            argstr='-applyxfm',
            usedefault=True,
        ),
        args=dict(argstr='%s', ),
        bbrslope=dict(
            argstr='-bbrslope %f',
            min_ver='5.0.0',
        ),
        bbrtype=dict(
            argstr='-bbrtype %s',
            min_ver='5.0.0',
        ),
        bgvalue=dict(argstr='-setbackground %f', ),
        bins=dict(argstr='-bins %d', ),
        coarse_search=dict(
            argstr='-coarsesearch %d',
            units='degrees',
        ),
        cost=dict(argstr='-cost %s', ),
        cost_func=dict(argstr='-searchcost %s', ),
        datatype=dict(argstr='-datatype %s', ),
        display_init=dict(argstr='-displayinit', ),
        dof=dict(argstr='-dof %d', ),
        echospacing=dict(
            argstr='-echospacing %f',
            min_ver='5.0.0',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fieldmap=dict(
            argstr='-fieldmap %s',
            extensions=None,
            min_ver='5.0.0',
        ),
        fieldmapmask=dict(
            argstr='-fieldmapmask %s',
            extensions=None,
            min_ver='5.0.0',
        ),
        fine_search=dict(
            argstr='-finesearch %d',
            units='degrees',
        ),
        force_scaling=dict(argstr='-forcescaling', ),
        in_file=dict(
            argstr='-in %s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        in_matrix_file=dict(
            argstr='-init %s',
            extensions=None,
        ),
        in_weight=dict(
            argstr='-inweight %s',
            extensions=None,
        ),
        interp=dict(argstr='-interp %s', ),
        min_sampling=dict(
            argstr='-minsampling %f',
            units='mm',
        ),
        no_clamp=dict(argstr='-noclamp', ),
        no_resample=dict(argstr='-noresample', ),
        no_resample_blur=dict(argstr='-noresampblur', ),
        no_search=dict(argstr='-nosearch', ),
        out_file=dict(
            argstr='-out %s',
            extensions=None,
            hash_files=False,
            name_source=['in_file'],
            name_template='%s_flirt',
            position=2,
        ),
        out_log=dict(
            extensions=None,
            keep_extension=True,
            name_source=['in_file'],
            name_template='%s_flirt.log',
            requires=['save_log'],
        ),
        out_matrix_file=dict(
            argstr='-omat %s',
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source=['in_file'],
            name_template='%s_flirt.mat',
            position=3,
        ),
        output_type=dict(),
        padding_size=dict(
            argstr='-paddingsize %d',
            units='voxels',
        ),
        pedir=dict(
            argstr='-pedir %d',
            min_ver='5.0.0',
        ),
        ref_weight=dict(
            argstr='-refweight %s',
            extensions=None,
        ),
        reference=dict(
            argstr='-ref %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        rigid2D=dict(argstr='-2D', ),
        save_log=dict(),
        schedule=dict(
            argstr='-schedule %s',
            extensions=None,
        ),
        searchr_x=dict(
            argstr='-searchrx %s',
            units='degrees',
        ),
        searchr_y=dict(
            argstr='-searchry %s',
            units='degrees',
        ),
        searchr_z=dict(
            argstr='-searchrz %s',
            units='degrees',
        ),
        sinc_width=dict(
            argstr='-sincwidth %d',
            units='voxels',
        ),
        sinc_window=dict(argstr='-sincwindow %s', ),
        uses_qform=dict(argstr='-usesqform', ),
        verbose=dict(argstr='-verbose %d', ),
        wm_seg=dict(
            argstr='-wmseg %s',
            extensions=None,
            min_ver='5.0.0',
        ),
        wmcoords=dict(
            argstr='-wmcoords %s',
            extensions=None,
            min_ver='5.0.0',
        ),
        wmnorms=dict(
            argstr='-wmnorms %s',
            extensions=None,
            min_ver='5.0.0',
        ),
    )
    inputs = ApplyXFM.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ApplyXFM_outputs():
    output_map = dict(
        out_file=dict(extensions=None, ),
        out_log=dict(extensions=None, ),
        out_matrix_file=dict(extensions=None, ),
    )
    outputs = ApplyXFM.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import FLAMEO


def test_FLAMEO_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        burnin=dict(argstr='--burnin=%d', ),
        cope_file=dict(
            argstr='--copefile=%s',
            extensions=None,
            mandatory=True,
        ),
        cov_split_file=dict(
            argstr='--covsplitfile=%s',
            extensions=None,
            mandatory=True,
        ),
        design_file=dict(
            argstr='--designfile=%s',
            extensions=None,
            mandatory=True,
        ),
        dof_var_cope_file=dict(
            argstr='--dofvarcopefile=%s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        f_con_file=dict(
            argstr='--fcontrastsfile=%s',
            extensions=None,
        ),
        fix_mean=dict(argstr='--fixmean', ),
        infer_outliers=dict(argstr='--inferoutliers', ),
        log_dir=dict(
            argstr='--ld=%s',
            usedefault=True,
        ),
        mask_file=dict(
            argstr='--maskfile=%s',
            extensions=None,
            mandatory=True,
        ),
        n_jumps=dict(argstr='--njumps=%d', ),
        no_pe_outputs=dict(argstr='--nopeoutput', ),
        outlier_iter=dict(argstr='--ioni=%d', ),
        output_type=dict(),
        run_mode=dict(
            argstr='--runmode=%s',
            mandatory=True,
        ),
        sample_every=dict(argstr='--sampleevery=%d', ),
        sigma_dofs=dict(argstr='--sigma_dofs=%d', ),
        t_con_file=dict(
            argstr='--tcontrastsfile=%s',
            extensions=None,
            mandatory=True,
        ),
        var_cope_file=dict(
            argstr='--varcopefile=%s',
            extensions=None,
        ),
    )
    inputs = FLAMEO.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FLAMEO_outputs():
    output_map = dict(
        copes=dict(),
        fstats=dict(),
        mrefvars=dict(),
        pes=dict(),
        res4d=dict(),
        stats_dir=dict(),
        tdof=dict(),
        tstats=dict(),
        var_copes=dict(),
        weights=dict(),
        zfstats=dict(),
        zstats=dict(),
    )
    outputs = FLAMEO.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
